_LANG_CONFIG = {
  "mya": {
    "fleurs_lang_code": "my",
    "fleurs_country_code": "mm"
  },
  "ceb": {
    "fleurs_lang_code": "ceb",
    "fleurs_country_code": "ph"
  },
  "fil": {
    "fleurs_lang_code": "fil",
    "fleurs_country_code": "ph"
  },
  "ind": {
    "fleurs_lang_code": "id",
    "fleurs_country_code": "id"
  },
  "jav": {
    "fleurs_lang_code": "jv",
    "fleurs_country_code": "id"
  },
  "khm": {
    "fleurs_lang_code": "km",
    "fleurs_country_code": "kh"
  },
  "lao": {
    "fleurs_lang_code": "lo",
    "fleurs_country_code": "la"
  },
  "zlm": {
    "fleurs_lang_code": "ms",
    "fleurs_country_code": "my"
  },
  "tha": {
    "fleurs_lang_code": "th",
    "fleurs_country_code": "th"
  },
  "vie": {
    "fleurs_lang_code": "vi",
    "fleurs_country_code": "vn"
  }
}



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
PFSA-ID is an annotated corpus for Public Figure Statement Attribution in the Indonesian Language.
The annotation using the multi-class named entity recognition with 11 labels: PERSON, ROLE, AFFILIATION, PERSONCOREF, CUE, CUECOREF, STATEMENT, ISSUE, EVENT, DATETIME, and LOCATION and using the BILOU scheme as the representation of tokens.
"""
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@article{PURNOMOWP2022,
  title = {PFSA-ID: an annotated Indonesian corpus and baseline model of public figures statements attributions},
  journal = {Global Knowledge, Memory and Communication},
  volume = {ahead-of-print},
  pages = {ahead-of-print},
  year = {2022},
  issn = {2514-9342},
  doi = {https://doi.org/10.1108/GKMC-04-2022-0091},
  url = {https://www.emerald.com/insight/content/doi/10.1108/GKMC-04-2022-0091/full/html},
  author = {Yohanes Sigit {Purnomo W.P.} and Yogan Jaya Kumar and Nur Zareen Zulkarnain},
  keywords = {Indonesian corpus, Public figures, Statement attribution, News article, Baseline model, Named entity recognition},
  abstract = {Purpose By far, the corpus for the quotation extraction and quotation attribution tasks in Indonesian is still limited in quantity and depth. This study aims to develop an Indonesian corpus of public figure statements attributions and a
    baseline model for attribution extraction, so it will contribute to fostering research in information extraction for the Indonesian language. Design/methodology/approach The methodology is divided into corpus development and extraction model
    development. During corpus development, data were collected and annotated. The development of the extraction model entails feature extraction, the definition of the model architecture, parameter selection and configuration,
    model training and evaluation, as well as model selection. Findings The Indonesian corpus of public figure statements attribution achieved 90.06% agreement level between the annotator and experts and could serve as a gold standard corpus.
    Furthermore, the baseline model predicted most labels and achieved 82.026% F-score. Originality/value To the best of the authors’ knowledge, the resulting corpus is the first corpus for attribution of public figures’ statements in the Indonesian
    language, which makes it a significant step for research on attribution extraction in the language. The resulting corpus and the baseline model can be used as a benchmark for further research. Other researchers could follow the methods presented
    in this paper to develop a new corpus and baseline model for other languages.
    }
}

@article{PurnomoWP2024,
  title = {Extraction and attribution of public figures statements for journalism in Indonesia using deep learning},
  volume = {289},
  ISSN = {0950-7051},
  url = {http://dx.doi.org/10.1016/j.knosys.2024.111558},
  DOI = {10.1016/j.knosys.2024.111558},
  journal = {Knowledge-Based Systems},
  publisher = {Elsevier BV},
  author = {Purnomo W.P.,  Yohanes Sigit and Kumar,  Yogan Jaya and Zulkarnain,  Nur Zareen and Raza,  Basit},
  year = {2024},
  month = apr,
  pages = {111558}
}
"""

_DATASETNAME = "pfsa_id"

_DESCRIPTION = """\
PFSA-ID is an annotated corpus for Public Figure Statement Attribution in the Indonesian Language.

The annotation using the multi-class named entity recognition with 11 labels: PERSON, ROLE, AFFILIATION, PERSONCOREF, CUE, CUECOREF, STATEMENT, ISSUE, EVENT, DATETIME, and LOCATION and using the BILOU scheme as the representation of tokens.
"""

_HOMEPAGE = "https://github.com/sigit-purnomo/pfsa-id"

_LANGUAGES = ["ind"]

_LICENSE = Licenses.CC_BY_NC_SA_4_0.value

_LOCAL = False

_URLS = {
    "pfsa_id": {
        "train": [
            "https://raw.githubusercontent.com/sigit-purnomo/pfsa-id-dl/main/corpus/pfsa-id/train-60.txt",
            "https://raw.githubusercontent.com/sigit-purnomo/pfsa-id-dl/main/corpus/pfsa-id/train-70.txt",
            "https://raw.githubusercontent.com/sigit-purnomo/pfsa-id-dl/main/corpus/pfsa-id/train-80.txt",
        ],
        "dev": [
            "https://raw.githubusercontent.com/sigit-purnomo/pfsa-id-dl/main/corpus/pfsa-id/dev-20.txt",
            "https://raw.githubusercontent.com/sigit-purnomo/pfsa-id-dl/main/corpus/pfsa-id/dev-30.txt",
            "https://raw.githubusercontent.com/sigit-purnomo/pfsa-id-dl/main/corpus/pfsa-id/dev-40.txt",
        ],
    },
    "pfsa_id_med": {
        "train": [
            "https://raw.githubusercontent.com/sigit-purnomo/pfsa-id-dl/main/corpus/pfsa-id-med/train-dl-60-v2.txt",
            "https://raw.githubusercontent.com/sigit-purnomo/pfsa-id-dl/main/corpus/pfsa-id-med/train-dl-70-v2.txt",
            "https://raw.githubusercontent.com/sigit-purnomo/pfsa-id-dl/main/corpus/pfsa-id-med/train-dl-80-v2.txt",
        ],
        "dev": [
            "https://raw.githubusercontent.com/sigit-purnomo/pfsa-id-dl/main/corpus/pfsa-id-med/dev-dl-20-v2.txt",
            "https://raw.githubusercontent.com/sigit-purnomo/pfsa-id-dl/main/corpus/pfsa-id-med/dev-dl-30-v2.txt",
            "https://raw.githubusercontent.com/sigit-purnomo/pfsa-id-dl/main/corpus/pfsa-id-med/dev-dl-40-v2.txt",
        ],
    },
    "pfsa_id_test": {
        "train": ["https://raw.githubusercontent.com/sigit-purnomo/pfsa-id-dl/main/corpus/pfsa-id-test/train.txt"],
        "dev": ["https://raw.githubusercontent.com/sigit-purnomo/pfsa-id-dl/main/corpus/pfsa-id-test/dev.txt"],
        "test": ["https://raw.githubusercontent.com/sigit-purnomo/pfsa-id-dl/main/corpus/pfsa-id-test/test-data.txt"],
    },
}

_SUPPORTED_TASKS = [Tasks.NAMED_ENTITY_RECOGNITION, Tasks.POS_TAGGING]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class PfsaIdDataset(datasets.GeneratorBasedBuilder):
    """PFSA-ID is an annotated corpus for Public Figure Statement Attribution in the Indonesian Language.

    The annotation using the multi-class named entity recognition with 11 labels: PERSON, ROLE, AFFILIATION, PERSONCOREF, CUE, CUECOREF, STATEMENT, ISSUE, EVENT, DATETIME, and LOCATION and using the BILOU scheme as the representation of tokens."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    LABEL_CLASSES = [
        "L-EVENT",
        "U-EVENT",
        "B-EVENT",
        "B-STATEMENT",
        "L-DATETIME",
        "I-ROLE",
        "B-DATETIME",
        "I-LOCATION",
        "I-STATEMENT",
        "B-PERSON",
        "U-AFFILIATION",
        "I-PERSONCOREF",
        "B-ROLE",
        "I-EVENT",
        "B-ISSUE",
        "B-AFFILIATION",
        "I-AFFILIATION",
        "U-PERSON",
        "U-ROLE",
        "B-LOCATION",
        "L-LOCATION",
        "L-ROLE",
        "U-CUE",
        "I-DATETIME",
        "L-PERSONCOREF",
        "L-AFFILIATION",
        "B-PERSONCOREF",
        "O",
        "I-ISSUE",
        "L-PERSON",
        "U-CUECOREF",
        "U-LOCATION",
        "L-STATEMENT",
        "U-PERSONCOREF",
        "I-PERSON",
        "L-ISSUE",
    ]

    SUBSETS = ["", "_med", "_test"]

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}{subset}_source",
            version=datasets.Version(_SOURCE_VERSION),
            description=f"{_DATASETNAME} source schema for pfsa_id{subset} subset",
            schema="source",
            subset_id=f"{_DATASETNAME}{subset}",
        )
        for subset in SUBSETS
    ] + [
        SEACrowdConfig(
            name=f"{_DATASETNAME}{subset}_seacrowd_seq_label",
            version=datasets.Version(_SEACROWD_VERSION),
            description=f"{_DATASETNAME} SEACrowd seq_label schema for pfsa_id{subset} subset",
            schema="seacrowd_seq_label",
            subset_id=f"{_DATASETNAME}{subset}",
        )
        for subset in SUBSETS
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "tokens": datasets.Sequence(datasets.Value("string")),
                    "labels": datasets.Sequence(datasets.ClassLabel(names=self.LABEL_CLASSES)),
                }
            )

        elif self.config.schema == "seacrowd_seq_label":
            features = schemas.seq_label_features(self.LABEL_CLASSES)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        urls = _URLS[self.config.subset_id]
        data_dir = dl_manager.download_and_extract(urls)

        split_gen = [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_dir["train"],
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": data_dir["dev"],
                },
            ),
        ]

        if self.config.subset_id == "pfsa_id_test":
            split_gen.append(
                datasets.SplitGenerator(
                    name=datasets.Split.TEST,
                    gen_kwargs={
                        "filepath": data_dir["test"],
                    },
                )
            )

        return split_gen

    def _generate_examples(self, filepath: Path) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        i = 0
        for _, path in enumerate(filepath):
            tokens, labels = [], []
            lines = open(path, "r").readlines()

            for line in lines:
                if line == "\n":
                    yield i, {
                        "id": str(i),
                        "tokens": tokens,
                        "labels": labels,
                    }
                    i += 1
                    tokens = []
                    labels = []
                else:
                    token, label = line.split("\t")
                    tokens.append(token)
                    labels.append(label.strip("\n"))



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
The Malaysia AI Hansard Scrape dataset contains 142,766 PDFs from the Malaysian Parliament website.
"""

from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@article{malaysua_ai_hansard,
  author    = {{Malaysia-AI}},
  title     = {Crawl Malaysian Hansard},
  year      = {2023},  % Change to the relevant year if known
  url       = {https://huggingface.co/datasets/malaysia-ai/crawl-malaysian-hansard}

}
"""

_DATASETNAME = "malaysia_ai_hansard"

_DESCRIPTION = """\
The Malaysia AI Hansard Scrape dataset contains 142,766 PDFs from the Malaysian Parliament website.
(https://www.parlimen.gov.my/hansard-dewan-rakyat.html?uweb=dr).
It includes a JSON file for each document with the text labeled "original", page numbers "no_page" and "actual_no_page", the document's "date", and the "url" of the original PDF.
"""

_HOMEPAGE = "https://huggingface.co/datasets/malaysia-ai/crawl-malaysian-hansard"

_LANGUAGES = ["zlm"]

_LICENSE = Licenses.APACHE_2_0.value

_LOCAL = False


_URLS = {
    _DATASETNAME: "https://huggingface.co/datasets/malaysia-ai/crawl-malaysian-hansard",
}


_SUPPORTED_TASKS = [Tasks.SELF_SUPERVISED_PRETRAINING]
_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class MalaysiaAIHansardDataset(datasets.GeneratorBasedBuilder):
    """Malaysia AI Hansard Scrape dataset contains 142,766 PDFs from the Malaysian Parliament website."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="malaysia_ai_hansard_source",
            version=SOURCE_VERSION,
            description="malaysia_ai_hansard source schema",
            schema="source",
            subset_id="malaysia_ai_hansard",
        ),
        SEACrowdConfig(
            name="malaysia_ai_hansard_seacrowd_ssp",
            version=SEACROWD_VERSION,
            description="malaysia_ai_hansard SEACrowd schema",
            schema="seacrowd_ssp",
            subset_id="malaysia_ai_hansard",
        ),
    ]

    DEFAULT_CONFIG_NAME = "malaysia_ai_hansard_source"

    def _info(self) -> datasets.DatasetInfo:
        # Create the source schema; this schema will keep all keys/information/labels as close to the original dataset as possible.

        # You can arbitrarily nest lists and dictionaries.
        # For iterables, use lists over tuples or `datasets.Sequence`

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "original": datasets.Value("string"),
                    "cleaned": datasets.Value("string"),
                    "no_page": datasets.Value("string"),
                    "actual_no_page": datasets.Value("string"),
                    "date": datasets.Value("string"),
                    "url": datasets.Value("string"),
                }
            )

        elif self.config.schema == "seacrowd_ssp":
            features = schemas.ssp_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""

        urls = _URLS[_DATASETNAME]
        # data_dir = dl_manager.download_and_extract(urls)
        # dl_manager not used since dataloader uses HF 'load_dataset'

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": urls,
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""

        data = datasets.load_dataset("/".join(filepath.split("/")[-2:]), split="train")

        for key, sample in enumerate(data):
            if self.config.schema == "source":
                yield key, {
                    "original": sample["original"],
                    "cleaned": sample["cleaned"],
                    "no_page": sample["no_page"],
                    "actual_no_page": sample["actual_no_page"],
                    "date": sample["date"],
                    "url": sample["url"],
                }

            elif self.config.schema == "seacrowd_ssp":
                yield key, {
                    "id": key,
                    "text": sample["cleaned"],
                }


if __name__ == "__main__":
    datasets.load_dataset(__file__)



from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import pandas as pd

from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks
from seacrowd.utils import schemas

_CITATION = """\
@misc{lopo2024constructing,
      title={Constructing and Expanding Low-Resource and Underrepresented Parallel Datasets for Indonesian Local Languages}, 
      author={Joanito Agili Lopo and Radius Tanone},
      year={2024},
      eprint={2404.01009},
      archivePrefix={arXiv},
      primaryClass={cs.CL}
}
"""

_DATASETNAME = "bhinneka_korpus"
_DESCRIPTION = """The Bhinneka Korpus dataset was parallel dataset for five Indonesian Local Languages conducted 
through a volunteer-driven translation strategy, encompassing sentences in the Indonesian-English pairs and lexical 
terms. The dataset consist of parallel data with 16,000 sentences in total, details with 4,000 sentence pairs for two 
Indonesia local language and approximately 3,000 sentences for other languages, and one lexicon dataset creation for 
Beaye language. In addition, since beaye is a undocumented language, we don't have any information yet about the use 
of language code. Therefore, we used "day" (a code for land dayak language family) to represent the language."""

_HOMEPAGE = "https://github.com/joanitolopo/bhinneka-korpus"
_LICENSE = Licenses.APACHE_2_0.value
_URLS = "https://raw.githubusercontent.com/joanitolopo/bhinneka-korpus/main/"
_SUPPORTED_TASKS = [Tasks.MACHINE_TRANSLATION]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"

_LANGUAGES = ["abs", "aoz", "day", "mak", "mkn"]
LANGUAGES_TO_FILENAME_MAP = {
    "abs": "ambonese-malay",
    "aoz": "uab-meto",
    "day": "beaye",
    "mak": "makassarese",
    "mkn": "kupang-malay",
}

_LOCAL = False


class BhinnekaKorpusDataset(datasets.GeneratorBasedBuilder):
    """A Collection of Multilingual Parallel Datasets for 5 Indonesian Local Languages."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)
    SEACROWD_SCHEMA_NAME = "t2t"

    dataset_names = sorted([f"{_DATASETNAME}_{lang}" for lang in _LANGUAGES])
    BUILDER_CONFIGS = []
    for name in dataset_names:
        source_config = SEACrowdConfig(
            name=f"{name}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=name
        )
        BUILDER_CONFIGS.append(source_config)
        seacrowd_config = SEACrowdConfig(
            name=f"{name}_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=name
        )
        BUILDER_CONFIGS.append(seacrowd_config)

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_day_source"

    def _info(self) -> datasets.DatasetInfo:
        schema = self.config.schema
        features = datasets.Features(
            {
                "source_sentence": datasets.Value("string"),
                "target_sentence": datasets.Value("string"),
                "source_lang": datasets.Value("string"),
                "target_lang": datasets.Value("string")
            } if schema == "source" else schemas.text2text_features
            if schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}" else None
        )
        if features is None:
            raise ValueError("Invalid config schema")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        data_dir = []
        lang = self.config.name.split("_")[2]
        if lang in _LANGUAGES:
            data_dir.append(Path(dl_manager.download(_URLS + f"{LANGUAGES_TO_FILENAME_MAP[lang]}/{lang}.xlsx")))
        else:
            raise ValueError("Invalid language name")
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_dir[0],
                    "split": "train",
                    "language": lang
                }
            )
        ]

    def _generate_examples(self, filepath: Path, split: str, language: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        dfs = pd.read_excel(filepath, index_col=0, engine="openpyxl")
        source_sents = dfs["ind"]
        target_sents = dfs[language]

        for idx, (source, target) in enumerate(zip(source_sents.values, target_sents.values)):
            if self.config.schema == "source":
                example = {
                    "source_sentence": source,
                    "target_sentence": target,
                    "source_lang": "ind",
                    "target_lang": language
                }
            elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
                example = {
                    "id": str(idx),
                    "text_1": source,
                    "text_2": target,
                    "text_1_name": "ind",
                    "text_2_name": language,
                }
            yield idx, example



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
A dataset of Indonesian News for Named-Entity Recognition task.
  This dataset re-annotated the dataset previously provided by Syaifudin & Nurwidyantoro (2016)
  (https://github.com/yusufsyaifudin/Indonesia-ner) with a more standardized NER tags.
  There are three subsets, namely train.txt, dev.txt, and test.txt.
  Each file consists of three columns which are Tokens, PoS Tag, and NER Tag respectively.
  The format is following CoNLL dataset. The NER tag use the IOB format.
  The PoS tag using UDPipe (http://ufal.mff.cuni.cz/udpipe),
  a pipeline for tokenization, tagging, lemmatization and dependency parsing
  whose model is trained on UD Treebanks.
"""

from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@inproceedings{khairunnisa-etal-2020-towards,
    title = "Towards a Standardized Dataset on {I}ndonesian Named Entity Recognition",
    author = "Khairunnisa, Siti Oryza  and
      Imankulova, Aizhan  and
      Komachi, Mamoru",
    editor = "Shmueli, Boaz  and
      Huang, Yin Jou",
    booktitle = "Proceedings of the 1st Conference of the Asia-Pacific Chapter of the Association for Computational Linguistics
      and the 10th International Joint Conference on Natural Language Processing: Student Research Workshop",
    month = dec,
    year = "2020",
    address = "Suzhou, China",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/2020.aacl-srw.10",
    pages = "64--71",
    abstract = "In recent years, named entity recognition (NER) tasks in the Indonesian language
    have undergone extensive development. There are only a few corpora for Indonesian NER;
    hence, recent Indonesian NER studies have used diverse datasets. Although an open dataset is available,
    it includes only approximately 2,000 sentences and contains inconsistent annotations,
    thereby preventing accurate training of NER models without reliance on pre-trained models.
    Therefore, we re-annotated the dataset and compared the two annotations{'} performance
    using the Bidirectional Long Short-Term Memory and Conditional Random Field (BiLSTM-CRF) approach.
    Fixing the annotation yielded a more consistent result for the organization tag and improved the prediction score
    by a large margin. Moreover, to take full advantage of pre-trained models, we compared different feature embeddings
    to determine their impact on the NER task for the Indonesian language.",
}
"""

_DATASETNAME = "idner_news_2k"

_DESCRIPTION = """\
A dataset of Indonesian News for Named-Entity Recognition task.
  This dataset re-annotated the dataset previously provided by Syaifudin & Nurwidyantoro (2016)
  (https://github.com/yusufsyaifudin/Indonesia-ner) with a more standardized NER tags.
  There are three subsets, namely train.txt, dev.txt, and test.txt.
  Each file consists of three columns which are Tokens, PoS Tag, and NER Tag respectively.
  The format is following CoNLL dataset. The NER tag use the IOB format.
  The PoS tag using UDPipe (http://ufal.mff.cuni.cz/udpipe),
  a pipeline for tokenization, tagging, lemmatization and dependency parsing
  whose model is trained on UD Treebanks.
"""

_HOMEPAGE = "https://github.com/khairunnisaor/idner-news-2k"

_LANGUAGES = ["ind"]

_LICENSE = Licenses.MIT.value

_LOCAL = False

_URLS = {
    _DATASETNAME: {
        "train": "https://raw.githubusercontent.com/khairunnisaor/idner-news-2k/main/train.txt",
        "dev": "https://raw.githubusercontent.com/khairunnisaor/idner-news-2k/main/dev.txt",
        "test": "https://raw.githubusercontent.com/khairunnisaor/idner-news-2k/main/test.txt",
    },
}

_SUPPORTED_TASKS = [Tasks.NAMED_ENTITY_RECOGNITION]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class IdNerNews2kDataset(datasets.GeneratorBasedBuilder):
    """This dataset is designed for Named-Entity Recognition NLP task in Indonesian,
    consisting of train, dev, and test files in CoNLL format. The NER tag in IOB format."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    SEACROWD_SCHEMA_NAME = "seq_label"

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        NAMED_ENTITIES = ["B-LOC", "I-LOC", "B-ORG", "I-ORG", "B-PER", "I-PER", "O"]
        POS_TAGS = ["PROPN", "AUX", "NUM", "NOUN", "ADP", "PRON", "VERB", "ADV", "ADJ", "PUNCT", "DET", "PART", "SCONJ", "CCONJ", "SYM", "X"]

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "tokens": datasets.Sequence(datasets.Value("string")),
                    "pos_tags": datasets.Sequence(datasets.ClassLabel(names=POS_TAGS)),
                    "ner_tags": datasets.Sequence(datasets.ClassLabel(names=NAMED_ENTITIES)),
                }
            )

        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            features = schemas.seq_label.features(NAMED_ENTITIES)

        else:
            raise ValueError(f"Invalid config: {self.config.name}")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:

        urls = _URLS[_DATASETNAME]
        train_path = dl_manager.download_and_extract(urls["train"])
        dev_path = dl_manager.download_and_extract(urls["dev"])
        test_path = dl_manager.download_and_extract(urls["test"])

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": train_path,
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": test_path,
                    "split": "test",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": dev_path,
                    "split": "dev",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""

        df = pd.read_csv(filepath, delimiter=" ", header=None, skip_blank_lines=False)
        if self.config.schema == "source":
            tokens, pos_tags, ner_tags = [], [], []

            for idx, row in df.iterrows():
                if pd.isnull(row[0]):
                    if tokens:
                        yield idx, {"id": idx, "tokens": tokens, "pos_tags": pos_tags, "ner_tags": ner_tags}
                        tokens, pos_tags, ner_tags = [], [], []
                else:
                    tokens.append(row[0])
                    pos_tags.append(row[1])
                    ner_tags.append(row[2])

        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            tokens, ner_tags = [], []

            for idx, row in df.iterrows():
                if pd.isnull(row[0]):
                    if tokens:
                        yield idx, {"id": idx, "tokens": tokens, "labels": ner_tags}
                        tokens, ner_tags = [], []
                else:
                    tokens.append(row[0])
                    ner_tags.append(row[2])
        else:
            raise ValueError(f"Invalid config: {self.config.name}")


import itertools
from pathlib import Path
from typing import List, Tuple

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_DATASETNAME = "parallel_asian_treebank"

_LANGUAGES = ["khm", "lao", "mya", "ind", "fil", "zlm", "tha", "vie"]
_LANGUAGES_TO_FILENAME_LANGUAGE_CODE = {
    "khm": "khm",
    "lao": "lo",
    "mya": "my",
    "ind": "id",
    "fil": "fil",
    "zlm": "ms",
    "tha": "th",
    "vie": "vi",
    "eng": "en",
    "hin": "hi",
    "jpn": "ja",
    "zho": "zh",
}
_LOCAL = False
_CITATION = """\
@inproceedings{riza2016introduction,
  title={Introduction of the asian language treebank},
  author={Riza, Hammam and Purwoadi, Michael and Uliniansyah, Teduh and Ti, Aw Ai and Aljunied, Sharifah Mahani and Mai, Luong Chi and Thang, Vu Tat and Thai, Nguyen Phuong and Chea, Vichet and Sam, Sethserey and others},
  booktitle={2016 Conference of The Oriental Chapter of International Committee for Coordination and Standardization of Speech Databases and Assessment Techniques (O-COCOSDA)},
  pages={1--6},
  year={2016},
  organization={IEEE}
}
"""

_DESCRIPTION = """\
The ALT project aims to advance the state-of-the-art Asian natural language processing (NLP) techniques through the open collaboration for developing and using ALT.
It was first conducted by NICT and UCSY as described in Ye Kyaw Thu, Win Pa Pa, Masao Utiyama, Andrew Finch and Eiichiro Sumita (2016).
Then, it was developed under ASEAN IVO.
The process of building ALT began with sampling about 20,000 sentences from English Wikinews, and then these sentences were translated into the other languages.
ALT now has 13 languages: Bengali, English, Filipino, Hindi, Bahasa Indonesia, Japanese, Khmer, Lao, Malay, Myanmar (Burmese), Thai, Vietnamese, Chinese (Simplified Chinese).
"""

_HOMEPAGE = "https://www2.nict.go.jp/astrec-att/member/mutiyama/ALT/"

_LICENSE = Licenses.CC_BY_4_0.value

_URLS = {
    "data": "https://www2.nict.go.jp/astrec-att/member/mutiyama/ALT/ALT-Parallel-Corpus-20191206.zip",
    "train": "https://www2.nict.go.jp/astrec-att/member/mutiyama/ALT/URL-train.txt",
    "dev": "https://www2.nict.go.jp/astrec-att/member/mutiyama/ALT/URL-dev.txt",
    "test": "https://www2.nict.go.jp/astrec-att/member/mutiyama/ALT/URL-test.txt",
}

_SUPPORTED_TASKS = [Tasks.MACHINE_TRANSLATION]

_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class ParallelAsianTreebankDataset(datasets.GeneratorBasedBuilder):
    """The ALT project aims to advance the state-of-the-art Asian natural language processing (NLP) techniques through the open collaboration for developing and using ALT"""

    BUILDER_CONFIGS = []
    lang_combinations = list(itertools.combinations(_LANGUAGES_TO_FILENAME_LANGUAGE_CODE.keys(), 2))
    for lang_a, lang_b in lang_combinations:
        if lang_a not in _LANGUAGES and lang_b not in _LANGUAGES:
            # Don't create a subset if both languages are not from SEA
            pass
        else:
            BUILDER_CONFIGS.append(
                SEACrowdConfig(
                    name=f"{_DATASETNAME}_{lang_a}_{lang_b}_source",
                    version=_SOURCE_VERSION,
                    description=f"{_DATASETNAME} source schema",
                    schema="source",
                    subset_id=f"{_DATASETNAME}_{lang_a}_{lang_b}_source",
                )
            )
            BUILDER_CONFIGS.append(
                SEACrowdConfig(
                    name=f"{_DATASETNAME}_{lang_a}_{lang_b}_seacrowd_t2t",
                    version=_SOURCE_VERSION,
                    description=f"{_DATASETNAME} seacrowd schema",
                    schema="seacrowd_t2t",
                    subset_id=f"{_DATASETNAME}_{lang_a}_{lang_b}_seacrowd_t2t",
                )
            )

    def _info(self):
        # The features are the same for both source and seacrowd
        features = schemas.text2text_features
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:

        def _split_at_n(text: str, n: int) -> Tuple[str, str]:
            """Split text on the n-th instance"""
            return ("_".join(text.split("_")[:n]), "_".join(text.split("_")[n:]))

        _, subset = _split_at_n(self.config.subset_id, 3)
        lang_pair, _ = _split_at_n(subset, 2)
        lang_a, lang_b = lang_pair.split("_")

        data_dir = Path(dl_manager.download_and_extract(_URLS["data"])) / "ALT-Parallel-Corpus-20191206"

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"data_dir": data_dir, "lang_a": lang_a, "lang_b": lang_b, "split_file": dl_manager.download(_URLS["train"])},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"data_dir": data_dir, "lang_a": lang_a, "lang_b": lang_b, "split_file": dl_manager.download(_URLS["test"])},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"data_dir": data_dir, "lang_a": lang_a, "lang_b": lang_b, "split_file": dl_manager.download(_URLS["dev"])},
            ),
        ]

    def _generate_examples(self, data_dir: Path, lang_a: str, lang_b: str, split_file: str):

        def _get_texts(lang: str) -> pd.DataFrame:
            with open(data_dir / f"data_{_LANGUAGES_TO_FILENAME_LANGUAGE_CODE[lang]}.txt", "r") as f:
                rows = [line.strip().split("\t") for line in f.readlines()]

            url_id = [row[0].split(".")[1] for row in rows]
            sent_id = [row[0].split(".")[-1] for row in rows]
            text = []
            for row in rows:
                # There are rows with an empty text, but they are still tagged with an ID
                # so we keep them and just pass an empty string.
                sent = row[1] if len(row) > 1 else ""
                text.append(sent)


            df = pd.DataFrame({"url_id": url_id, "sent_id": sent_id, "text": text})
            return df

        with open(split_file, "r") as f:
            url_texts = [line.strip() for line in f.readlines()]
            # Get valid URLs for the split
            urlids_for_current_split = [row.split("\t")[0].split(".")[1] for row in url_texts]

        lang_a_df = _get_texts(lang_a)
        lang_b_df = _get_texts(lang_b)

        for idx, urlid in enumerate(urlids_for_current_split):
            lang_a_df_split = lang_a_df[lang_a_df["url_id"] == urlid]
            lang_b_df_split = lang_b_df[lang_b_df["url_id"] == urlid]

            if len(lang_a_df_split) == 0 or len(lang_b_df_split) == 0:
                # Sometimes, not all languages have values for a specific ID
                pass
            else:
                text_a = " ".join(lang_a_df_split["text"].to_list())
                text_b = " ".join(lang_b_df_split["text"].to_list())

                # Same schema for both source and SEACrowd
                yield idx, {
                    "id": idx,
                    "text_1": text_a,
                    "text_2": text_b,
                    "text_1_name": lang_a,
                    "text_2_name": lang_b,
                }



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Contains 110 large-scale ground-truth bilingual dictionaries created and released by Meta using an internal translation tool.
The dictionaries account for polysemy. The data comprises of a train and test split of 5000 and 1500 unique source words, as well as a larger set of up to 100k pairs.
It comprises of Europeans languages in every direction, and SEA languages to and from English.
"""

from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@inproceedings{lample2018word,
  title={Word translation without parallel data},
  author={Lample, Guillaume and Conneau, Alexis and Ranzato, Marc'Aurelio and Denoyer, Ludovic and J{\'e}gou, Herv{\'e}},
  booktitle={International Conference on Learning Representations},
  year={2018}}
}
"""

_DATASETNAME = "muse"

_DESCRIPTION = """\
Contains 110 large-scale ground-truth bilingual dictionaries created and released by Meta using an internal translation tool.
The dictionaries account for polysemy. The data comprises of a train and test split of 5000 and 1500 unique source words, as well as a larger set of up to 100k pairs.
It comprises of Europeans languages in every direction, and SEA languages to and from English.
"""

_HOMEPAGE = "https://github.com/facebookresearch/MUSE#ground-truth-bilingual-dictionaries"

_LANGUAGES = ["tgl", "ind", "zlm", "tha", "vie"]

_LICENSE = Licenses.CC_BY_NC_ND_4_0.value

_LOCAL = False

_TRAIN_URL_TEMPLATE = "https://dl.fbaipublicfiles.com/arrival/dictionaries/{src}-{tgt}.0-5000.txt"
_TEST_URL_TEMPLATE = "https://dl.fbaipublicfiles.com/arrival/dictionaries/{src}-{tgt}.5000-6500.txt"

_SUPPORTED_TASKS = [Tasks.MACHINE_TRANSLATION]

_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"

configs = {
    "tgl": ["eng"],
    "ind": ["eng"],
    "zlm": ["eng"],
    "tha": ["eng"],
    "vie": ["eng"],
    "eng": ["tha", "vie", "tgl", "zlm", "ind"],
}

langid_dict = {
    "eng": "en",
    "tgl": "tl",
    "ind": "id",
    "zlm": "ms",
    "tha": "th",
    "vie": "vi",
}


class MUSEDataset(datasets.GeneratorBasedBuilder):
    """Large-scale ground-truth bilingual dictionaries"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = (
        [
            SEACrowdConfig(
                name=f"{_DATASETNAME}_source",
                version=SOURCE_VERSION,
                description=f"{_DATASETNAME} source schema",
                schema="source",
                subset_id=f"{_DATASETNAME}_tgl_eng",
            ),
            SEACrowdConfig(
                name=f"{_DATASETNAME}_seacrowd_t2t",
                version=SEACROWD_VERSION,
                description=f"{_DATASETNAME} SEACrowd schema",
                schema="seacrowd_t2t",
                subset_id=f"{_DATASETNAME}_tgl_eng",
            ),
        ]
        + [
            SEACrowdConfig(
                name=f"{_DATASETNAME}_{src_lang}_{tgt_lang}_source",
                version=datasets.Version(_SOURCE_VERSION),
                description=f"{_DATASETNAME} source schema",
                schema="source",
                subset_id=f"{_DATASETNAME}_{src_lang}_{tgt_lang}",
            )
            for src_lang in configs
            for tgt_lang in configs[src_lang]
        ]
        + [
            SEACrowdConfig(
                name=f"{_DATASETNAME}_{src_lang}_{tgt_lang}_seacrowd_t2t",
                version=datasets.Version(_SOURCE_VERSION),
                description=f"{_DATASETNAME} SEACrowd schema",
                schema="seacrowd_t2t",
                subset_id=f"{_DATASETNAME}_{src_lang}_{tgt_lang}",
            )
            for src_lang in configs
            for tgt_lang in configs[src_lang]
        ]
    )

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "src_text": datasets.Value("string"),
                    "tgt_text": datasets.Value("string"),
                }
            )

        elif self.config.schema == "seacrowd_t2t":
            features = schemas.text2text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:

        _, src_lang, tgt_lang = self.config.subset_id.split("_")
        train_url = _TRAIN_URL_TEMPLATE.format(src=langid_dict[src_lang], tgt=langid_dict[tgt_lang])
        test_url = _TEST_URL_TEMPLATE.format(src=langid_dict[src_lang], tgt=langid_dict[tgt_lang])

        train_file = dl_manager.download_and_extract(train_url)
        test_file = dl_manager.download_and_extract(test_url)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "src_lang": src_lang,
                    "tgt_lang": tgt_lang,
                    "filepath": train_file,
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "src_lang": src_lang,
                    "tgt_lang": tgt_lang,
                    "filepath": test_file,
                },
            ),
        ]

    def _generate_examples(self, src_lang: str, tgt_lang: str, filepath: Path) -> Tuple[int, Dict]:
        if self.config.schema == "source":
            for row_id, line in enumerate(open(filepath)):
                src_text, tgt_text = line.strip().split("\t")
                yield row_id, {"id": row_id, "src_text": src_text, "tgt_text": tgt_text}

        elif self.config.schema == "seacrowd_t2t":
            for row_id, line in enumerate(open(filepath)):
                src_text, tgt_text = line.strip().split("\t")
                yield row_id, {
                    "id": row_id,
                    "text_1": src_text,
                    "text_2": tgt_text,
                    "text_1_name": src_lang,
                    "text_2_name": tgt_lang,
                }



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks, Licenses, TASK_TO_SCHEMA, SCHEMA_TO_FEATURES

_CITATION = """\
@misc{li2023bactrianx,
      title={Bactrian-X : A Multilingual Replicable Instruction-Following Model with Low-Rank Adaptation}, 
      author={Haonan Li and Fajri Koto and Minghao Wu and Alham Fikri Aji and Timothy Baldwin},
      year={2023},
      eprint={2305.15011},
      archivePrefix={arXiv},
      primaryClass={cs.CL}
}
"""

_DATASETNAME = "bactrian_x"

_DESCRIPTION = """\
The Bactrain-X dataset is a collection of 3.4M instruction-response pairs in 52
languages, that are obtained by translating 67K English instructions (alpaca-52k
+ dolly-15k) into 51 languages using Google Translate API. The translated
instructions are then fed to ChatGPT (gpt-3.5-turbo) to obtain its natural
responses, resulting in 3.4M instruction-response pairs in 52 languages (52
languages x 67k instances = 3.4M instances). Human evaluations were conducted to
evaluate response quality for several languages, with those of interest to
SEACrowd being Burmese and Tagalog.
"""

_HOMEPAGE = "https://github.com/mbzuai-nlp/Bactrian-X"

_LANGUAGES = ["mya", "tgl", "ind", "khm", "tha", "vie"]

_LICENSE = Licenses.CC_BY_NC_4_0.value

_LOCAL = False

_BASE_URL = "https://huggingface.co/datasets/MBZUAI/Bactrian-X/resolve/main/data/{subset}.json.gz?download=true"
_SUBSETS = ["my", "tl", "id", "km", "th", "vi"]

_SUPPORTED_TASKS = [Tasks.INSTRUCTION_TUNING]
_SEACROWD_SCHEMA = f"seacrowd_{TASK_TO_SCHEMA[_SUPPORTED_TASKS[0]].lower()}"  # t2t

_SOURCE_VERSION = "1.0.1"

_SEACROWD_VERSION = "1.0.0"


class BactrianXDataset(datasets.GeneratorBasedBuilder):
    """A collection of translated instruction-response pairs, evaluated with ChatGPT and human."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = []
    for subset in _SUBSETS:
        BUILDER_CONFIGS += [
            SEACrowdConfig(
                name=f"{_DATASETNAME}_{subset}_source",
                version=SOURCE_VERSION,
                description=f"{_DATASETNAME} {subset} source schema",
                schema="source",
                subset_id=subset,
            ),
            SEACrowdConfig(
                name=f"{_DATASETNAME}_{subset}_{_SEACROWD_SCHEMA}",
                version=SEACROWD_VERSION,
                description=f"{_DATASETNAME} {subset} SEACrowd schema",
                schema=_SEACROWD_SCHEMA,
                subset_id=subset,
            ),
        ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_id_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "instruction": datasets.Value("string"),
                    "input": datasets.Value("string"),
                    "id": datasets.Value("string"),
                    "output": datasets.Value("string"),
                }
            )
        elif self.config.schema == _SEACROWD_SCHEMA:
            features = SCHEMA_TO_FEATURES[
                TASK_TO_SCHEMA[_SUPPORTED_TASKS[0]]
            ]  # text2text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        data_url = _BASE_URL.format(subset=self.config.name.split("_")[2])
        data_path = Path(dl_manager.download_and_extract(data_url))

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "data_path": data_path,
                },
            )
        ]

    def _generate_examples(self, data_path: Path) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        with open(data_path, "r", encoding="utf-8") as file:
            data = json.load(file)

        if self.config.schema == "source":
            for idx, example in enumerate(data):
                yield idx, {
                    "instruction": example["instruction"],
                    "input": example["input"],
                    "id": example["id"],
                    "output": example["output"],
                }
        elif self.config.schema == _SEACROWD_SCHEMA:
            for idx, example in enumerate(data):
                yield idx, {
                    "id": example["id"],
                    "text_1": f"Instruction: {example['instruction']}\nInput: {example['input']}",
                    "text_2": example["output"],
                    "text_1_name": "instruction + input",
                    "text_2_name": "output",
                }



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
The LTI LangID corpus is a dataset for language identification.
The most recent version, v5, contains training data for 1266 languages, and some (possibly very tiny) amount of text for a total of 1706 languages.
"""
import os
import subprocess
from glob import glob
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@inproceedings{brown2014non,
  title={Non-linear mapping for improved identification of 1300+ languages},
  author={Brown, Ralf D},
  booktitle={Proceedings of the 2014 Conference on Empirical Methods in Natural Language Processing (EMNLP)},
  pages={627--632},
  year={2014}
}
"""

_DATASETNAME = "lti_langid_corpus"

_DESCRIPTION = """\
The LTI LangID corpus is a dataset for language identification.
The most recent version, v5, contains training data for 1266 languages, and some (possibly very tiny) amount of text for a total of 1706 languages.
This dataloader can only be executed in a BASH environment at the moment. (See https://github.com/SEACrowd/seacrowd-datahub/pull/405)
"""

_HOMEPAGE = "https://www.cs.cmu.edu/~ralf/langid.html"

_LANGUAGES = [
    "ifa",
    "ace",
    "btm",
    "mqj",
    "mbs",
    "nbq",
    "gor",
    "zyp",
    "tbl",
    "kjp",
    "kmk",
    "kqe",
    "ptu",
    "blw",
    "ceb",
    "prf",
    "yva",
    "zlm",
    "bps",
    "tdt",
    "mya",
    "dgc",
    "lus",
    "wrs",
    "abx",
    "rgu",
    "aaz",
    "agn",
    "ccp",
    "cmn",
    "jav",
    "obo",
    "due",
    "msk",
    "xsb",
    "syb",
    "ind",
    "lbk",
    "min",
    "smk",
    "att",
    "nod",
    "tdj",
    "atb",
    "atd",
    "pag",
    "hvn",
    "ksc",
    "lao",
    "kkl",
    "lti",
    "dao",
    "cgc",
    "tbk",
    "gdg",
    "amk",
    "mbd",
    "clu",
    "msb",
    "sbl",
    "cek",
    "khm",
    "yue",
    "sgb",
    "beu",
    "eip",
    "ifu",
    "mnw",
    "suc",
    "bgs",
    "pam",
    "ebk",
    "eng",
    "nfa",
    "cbk",
    "ify",
    "csy",
    "heg",
    "shn",
    "mta",
    "mbt",
    "lex",
    "tha",
    "mmn",
    "sun",
    "vie",
    "llg",
    "xnn",
    "txq",
    "bcl",
    "kje",
    "kne",
    "san",
    "hlt",
    "kyu",
    "bkd",
    "duo",
    "tet",
    "ury",
    "yka",
    "bjn",
    "tiy",
    "ivv",
    "agt",
    "ban",
    "blz",
    "mbi",
    "ilo",
    "mkn",
    "isd",
    "cth",
    "bpr",
    "por",
    "mbb",
    "tgl",
    "msm",
    "ivb",
    "tam",
    "plw",
    "alp",
    "row",
]

_ALL_LANGS = [
    "aai",
    "aak",
    "aau",
    "aaz",
    "abk",
    "abt",
    "abx",
    "aby",
    "aca",
    "acc",
    "ace",
    "acf",
    "acr",
    "acu",
    "adj",
    "ady",
    "adz",
    "aer",
    "aey",
    "afr",
    "agd",
    "agg",
    "agm",
    "agn",
    "agr",
    "agt",
    "agu",
    "ahr",
    "aia",
    "aii",
    "ake",
    "akh",
    "alb",
    "alp",
    "als",
    "aly",
    "ame",
    "amf",
    "amh",
    "ami",
    "amk",
    "amm",
    "amn",
    "amo",
    "amp",
    "amr",
    "amu",
    "amx",
    "ang",
    "anh",
    "anp",
    "anv",
    "aoi",
    "aoj",
    "aom",
    "aon",
    "apb",
    "ape",
    "apn",
    "apr",
    "apu",
    "apw",
    "apy",
    "apz",
    "ara",
    "arb",
    "arc",
    "are",
    "arg",
    "arl",
    "arn",
    "arp",
    "arq",
    "ary",
    "arz",
    "ase",
    "asm",
    "aso",
    "ast",
    "ata",
    "atb",
    "atd",
    "atg",
    "atj",
    "att",
    "aty",
    "auc",
    "aui",
    "auy",
    "ava",
    "avk",
    "avt",
    "awa",
    "awb",
    "awk",
    "awx",
    "aym",
    "ayp",
    "azb",
    "azd",
    "aze",
    "azg",
    "azz",
    "bak",
    "ban",
    "bao",
    "bar",
    "bba",
    "bbb",
    "bbr",
    "bch",
    "bcl",
    "bco",
    "bdd",
    "bdv",
    "bea",
    "bef",
    "bel",
    "ben",
    "beo",
    "ber",
    "beu",
    "bfb",
    "bfz",
    "bgc",
    "bgn",
    "bgs",
    "bgt",
    "bgw",
    "bhd",
    "bhg",
    "bhl",
    "bho",
    "bht",
    "big",
    "bjk",
    "bjn",
    "bjp",
    "bjr",
    "bjv",
    "bjz",
    "bkd",
    "bki",
    "bkq",
    "bla",
    "blw",
    "blz",
    "bmh",
    "bmr",
    "bmu",
    "bnp",
    "boa",
    "bod",
    "boj",
    "bon",
    "bos",
    "box",
    "bpr",
    "bps",
    "bpy",
    "bqc",
    "bqi",
    "bqj",
    "bqp",
    "bre",
    "bsj",
    "bsn",
    "bsp",
    "bss",
    "btm",
    "buk",
    "bul",
    "bus",
    "bvd",
    "bvr",
    "bwd",
    "bwo",
    "bxh",
    "bxr",
    "byr",
    "byx",
    "bzd",
    "bzh",
    "bzj",
    "caa",
    "cab",
    "cac",
    "caf",
    "cak",
    "cao",
    "cap",
    "car",
    "cat",
    "cav",
    "cax",
    "cbc",
    "cbi",
    "cbk",
    "cbm",
    "cbr",
    "cbs",
    "cbt",
    "cbu",
    "cbv",
    "cco",
    "ccp",
    "cdh",
    "cdj",
    "cdo",
    "ceb",
    "ceg",
    "cek",
    "ces",
    "cgc",
    "cha",
    "chd",
    "che",
    "chf",
    "chk",
    "chq",
    "chr",
    "chu",
    "chv",
    "chz",
    "cjo",
    "cjv",
    "ckb",
    "cke",
    "cki",
    "ckw",
    "cle",
    "clu",
    "cly",
    "cme",
    "cmn",
    "cni",
    "cnl",
    "cnr",
    "cnt",
    "coe",
    "cof",
    "cok",
    "con",
    "cop",
    "cor",
    "cos",
    "cot",
    "cox",
    "cpa",
    "cpb",
    "cpc",
    "cpu",
    "cpy",
    "crh",
    "crm",
    "crn",
    "cro",
    "crx",
    "csb",
    "cso",
    "csy",
    "cta",
    "cth",
    "cti",
    "cto",
    "ctp",
    "ctu",
    "cub",
    "cuc",
    "cui",
    "cuk",
    "cul",
    "cut",
    "cux",
    "cwe",
    "cya",
    "cym",
    "daa",
    "dad",
    "dag",
    "dah",
    "dan",
    "dao",
    "ded",
    "des",
    "deu",
    "dgc",
    "dgo",
    "dgz",
    "dhi",
    "dhn",
    "dif",
    "dik",
    "din",
    "diq",
    "div",
    "djk",
    "djr",
    "dnj",
    "dob",
    "dop",
    "dov",
    "dsb",
    "dso",
    "dty",
    "dub",
    "due",
    "duo",
    "dwr",
    "dww",
    "dyi",
    "dzo",
    "ebk",
    "egl",
    "eip",
    "eka",
    "ekk",
    "eko",
    "ell",
    "emi",
    "emp",
    "eng",
    "enm",
    "enq",
    "epo",
    "eri",
    "ese",
    "esi",
    "esk",
    "ess",
    "est",
    "etr",
    "eus",
    "ewe",
    "ext",
    "eza",
    "faa",
    "fai",
    "fan",
    "fao",
    "far",
    "fas",
    "ffm",
    "fin",
    "fit",
    "for",
    "fra",
    "frp",
    "frr",
    "fry",
    "fue",
    "fuf",
    "fuh",
    "fur",
    "fuv",
    "gag",
    "gah",
    "gai",
    "gam",
    "gan",
    "gaq",
    "gas",
    "gaw",
    "gaz",
    "gbk",
    "gbl",
    "gbm",
    "gcr",
    "gdg",
    "gdn",
    "gdr",
    "geb",
    "gfk",
    "ghs",
    "gla",
    "gle",
    "glg",
    "glk",
    "glv",
    "gmv",
    "gng",
    "gnn",
    "gnw",
    "gof",
    "gok",
    "gom",
    "gor",
    "got",
    "gqr",
    "grc",
    "grn",
    "gsw",
    "gub",
    "guc",
    "guh",
    "gui",
    "guj",
    "gul",
    "gum",
    "gun",
    "guo",
    "gup",
    "gux",
    "gvc",
    "gvf",
    "gvn",
    "gvs",
    "gwi",
    "gym",
    "gyr",
    "hak",
    "hat",
    "hau",
    "haw",
    "hbo",
    "hch",
    "heb",
    "heg",
    "hif",
    "hin",
    "hix",
    "hla",
    "hlb",
    "hlt",
    "hmo",
    "hns",
    "hop",
    "hot",
    "hoy",
    "hrv",
    "hsb",
    "hto",
    "hub",
    "hui",
    "hun",
    "hus",
    "huu",
    "huv",
    "hva",
    "hvn",
    "hwc",
    "hye",
    "hyw",
    "ian",
    "ibo",
    "icr",
    "ido",
    "ifa",
    "ifu",
    "ify",
    "ign",
    "iii",
    "ikk",
    "ikw",
    "ile",
    "ilo",
    "imo",
    "ina",
    "inb",
    "ind",
    "inh",
    "ino",
    "iou",
    "ipi",
    "iqw",
    "iri",
    "isd",
    "isl",
    "isn",
    "ita",
    "ivb",
    "ivv",
    "iws",
    "ixl",
    "izz",
    "jaa",
    "jac",
    "jae",
    "jam",
    "jao",
    "jav",
    "jbo",
    "jic",
    "jid",
    "jiv",
    "jni",
    "jpn",
    "juy",
    "jvn",
    "kaa",
    "kab",
    "kaj",
    "kal",
    "kan",
    "kaq",
    "kat",
    "kaz",
    "kbc",
    "kbd",
    "kbh",
    "kbm",
    "kbp",
    "kbq",
    "kca",
    "kdc",
    "kde",
    "kdl",
    "kek",
    "ken",
    "kew",
    "kfs",
    "kfw",
    "kfx",
    "kfy",
    "kgf",
    "kgk",
    "kgp",
    "khm",
    "khn",
    "khs",
    "khw",
    "khz",
    "kij",
    "kik",
    "kin",
    "kip",
    "kiu",
    "kiz",
    "kjb",
    "kje",
    "kjo",
    "kjp",
    "kjs",
    "kkc",
    "kkl",
    "klt",
    "klv",
    "kmg",
    "kmh",
    "kmk",
    "kmo",
    "kms",
    "kmu",
    "kne",
    "knf",
    "knj",
    "knv",
    "kog",
    "koi",
    "kom",
    "kor",
    "kos",
    "kpf",
    "kpg",
    "kpj",
    "kpr",
    "kpw",
    "kpx",
    "kqa",
    "kqc",
    "kqe",
    "kqf",
    "kql",
    "kqw",
    "krc",
    "ksc",
    "ksd",
    "ksh",
    "ksj",
    "ksr",
    "ktj",
    "ktm",
    "kto",
    "kud",
    "kue",
    "kum",
    "kup",
    "kur",
    "kvg",
    "kvn",
    "kwd",
    "kwf",
    "kwi",
    "kwj",
    "kxp",
    "kxv",
    "kyc",
    "kyf",
    "kyg",
    "kyq",
    "kyu",
    "kyv",
    "kyz",
    "kze",
    "lac",
    "lad",
    "lao",
    "lat",
    "lbb",
    "lbf",
    "lbk",
    "lbm",
    "lcm",
    "leu",
    "lex",
    "lez",
    "lfn",
    "lgl",
    "lid",
    "lif",
    "lij",
    "lim",
    "lin",
    "lit",
    "liv",
    "lki",
    "lld",
    "llg",
    "lmo",
    "lrc",
    "ltg",
    "lti",
    "ltz",
    "lug",
    "luo",
    "lus",
    "luz",
    "lvs",
    "lww",
    "lzh",
    "lzz",
    "maa",
    "mah",
    "mai",
    "maj",
    "mal",
    "mam",
    "maq",
    "mar",
    "mau",
    "mav",
    "maz",
    "mbb",
    "mbc",
    "mbd",
    "mbh",
    "mbi",
    "mbj",
    "mbl",
    "mbs",
    "mbt",
    "mca",
    "mcb",
    "mcd",
    "mcf",
    "mco",
    "mcp",
    "mcq",
    "mcr",
    "mdf",
    "mdy",
    "med",
    "mee",
    "meh",
    "mek",
    "meq",
    "met",
    "meu",
    "mfy",
    "mgc",
    "mgh",
    "mhr",
    "mib",
    "mie",
    "mig",
    "mih",
    "mil",
    "mim",
    "min",
    "mio",
    "mir",
    "mit",
    "mix",
    "miy",
    "miz",
    "mjc",
    "mjl",
    "mkd",
    "mkj",
    "mkl",
    "mkn",
    "mks",
    "mle",
    "mlg",
    "mlh",
    "mlp",
    "mlt",
    "mmn",
    "mmo",
    "mmx",
    "mna",
    "mnc",
    "mni",
    "mnw",
    "moh",
    "mon",
    "mop",
    "mox",
    "mpd",
    "mpj",
    "mpm",
    "mpp",
    "mps",
    "mpt",
    "mpx",
    "mqb",
    "mqj",
    "mri",
    "mrj",
    "msb",
    "msc",
    "msk",
    "msm",
    "msy",
    "mta",
    "mti",
    "mto",
    "mux",
    "muy",
    "mva",
    "mvn",
    "mwc",
    "mwe",
    "mwf",
    "mwl",
    "mwp",
    "mxb",
    "mxp",
    "mxq",
    "mxt",
    "mxv",
    "mya",
    "myk",
    "myu",
    "myv",
    "myw",
    "myy",
    "mza",
    "mzi",
    "mzn",
    "nab",
    "naf",
    "nag",
    "nah",
    "nak",
    "nan",
    "nap",
    "nas",
    "nav",
    "nbq",
    "nca",
    "nce",
    "nch",
    "ncj",
    "ncl",
    "ncu",
    "nde",
    "ndg",
    "ndj",
    "nds",
    "new",
    "nfa",
    "nfr",
    "ngp",
    "ngu",
    "nhe",
    "nhg",
    "nhi",
    "nho",
    "nhr",
    "nhu",
    "nhw",
    "nhx",
    "nhy",
    "nif",
    "nii",
    "nin",
    "niy",
    "nko",
    "nld",
    "nlg",
    "nlx",
    "nmw",
    "nna",
    "nno",
    "nnq",
    "noa",
    "nob",
    "nod",
    "noi",
    "nop",
    "not",
    "nou",
    "nov",
    "npi",
    "npl",
    "nqo",
    "nsn",
    "nso",
    "nss",
    "nsu",
    "ntj",
    "ntp",
    "ntu",
    "nuy",
    "nvm",
    "nwi",
    "nya",
    "nys",
    "nyu",
    "obo",
    "oci",
    "okv",
    "olo",
    "omw",
    "ong",
    "ons",
    "ood",
    "opm",
    "ory",
    "oss",
    "ota",
    "ote",
    "otm",
    "otn",
    "otq",
    "ots",
    "oym",
    "pab",
    "pad",
    "pag",
    "pah",
    "pam",
    "pan",
    "pao",
    "pap",
    "pbb",
    "pbc",
    "pcd",
    "pdc",
    "peg",
    "pes",
    "pfl",
    "pib",
    "pio",
    "pir",
    "piu",
    "pjt",
    "pli",
    "pls",
    "plt",
    "plu",
    "plw",
    "pma",
    "pms",
    "pnb",
    "pnt",
    "pob",
    "poe",
    "poh",
    "poi",
    "pol",
    "pon",
    "por",
    "poy",
    "ppo",
    "pps",
    "prf",
    "pri",
    "prq",
    "pst",
    "ptp",
    "ptu",
    "pwg",
    "qub",
    "quc",
    "quf",
    "quh",
    "qul",
    "qup",
    "quy",
    "qvc",
    "qve",
    "qvh",
    "qvm",
    "qvn",
    "qvs",
    "qvw",
    "qvz",
    "qwh",
    "qxh",
    "qxl",
    "qxn",
    "qxo",
    "rai",
    "ram",
    "rap",
    "reg",
    "rgu",
    "rkb",
    "rmc",
    "rmn",
    "rmy",
    "roh",
    "ron",
    "roo",
    "rop",
    "row",
    "rro",
    "rtw",
    "rue",
    "ruf",
    "rug",
    "rup",
    "rus",
    "rwo",
    "sab",
    "sah",
    "san",
    "sat",
    "sbe",
    "sbk",
    "sbl",
    "sbs",
    "sby",
    "sch",
    "sck",
    "scn",
    "sco",
    "sdh",
    "seh",
    "sey",
    "sgb",
    "sgj",
    "sgs",
    "sgz",
    "shi",
    "shj",
    "shn",
    "shp",
    "sim",
    "sin",
    "sja",
    "skr",
    "sli",
    "slk",
    "sll",
    "slv",
    "sme",
    "smk",
    "smo",
    "sna",
    "snc",
    "snd",
    "snn",
    "snp",
    "sny",
    "som",
    "soq",
    "soy",
    "spa",
    "spl",
    "spm",
    "spp",
    "sps",
    "spy",
    "srd",
    "sri",
    "srm",
    "srn",
    "srp",
    "srq",
    "srx",
    "ssd",
    "ssg",
    "ssx",
    "stp",
    "stq",
    "sua",
    "suc",
    "sue",
    "sun",
    "sus",
    "suz",
    "swe",
    "swh",
    "swp",
    "sxb",
    "syb",
    "szl",
    "szy",
    "tac",
    "taj",
    "tam",
    "tar",
    "tat",
    "tau",
    "tav",
    "taw",
    "tay",
    "tbc",
    "tbf",
    "tbg",
    "tbk",
    "tbl",
    "tbo",
    "tbz",
    "tca",
    "tcf",
    "tcs",
    "tcy",
    "tcz",
    "tdj",
    "tdt",
    "tdx",
    "ted",
    "tee",
    "tel",
    "ter",
    "tet",
    "tew",
    "tfr",
    "tgk",
    "tgl",
    "tgo",
    "tgp",
    "tha",
    "thr",
    "tif",
    "tik",
    "tim",
    "tir",
    "tiw",
    "tiy",
    "tke",
    "tkr",
    "tku",
    "tlf",
    "tlh",
    "tly",
    "tmd",
    "tna",
    "tnc",
    "tnk",
    "tnn",
    "tnp",
    "toc",
    "tod",
    "tof",
    "toj",
    "tok",
    "ton",
    "too",
    "top",
    "tos",
    "tpi",
    "tpt",
    "tpx",
    "tpz",
    "trc",
    "trq",
    "trs",
    "trv",
    "tsn",
    "tso",
    "tsw",
    "ttc",
    "tte",
    "tuc",
    "tue",
    "tuf",
    "tuk",
    "tuo",
    "tur",
    "tvk",
    "tvt",
    "twi",
    "txq",
    "txu",
    "tyv",
    "tzc",
    "tze",
    "tzj",
    "tzm",
    "tzo",
    "tzs",
    "tzz",
    "ubr",
    "ubu",
    "udm",
    "udu",
    "uig",
    "ukr",
    "uli",
    "ulk",
    "unx",
    "upv",
    "ura",
    "urb",
    "urd",
    "uri",
    "urt",
    "urw",
    "ury",
    "usa",
    "usp",
    "uvh",
    "uvl",
    "uzb",
    "vah",
    "vai",
    "var",
    "vec",
    "vep",
    "vid",
    "vie",
    "viv",
    "vls",
    "vmy",
    "vol",
    "vro",
    "waj",
    "wal",
    "wap",
    "wat",
    "waw",
    "wbi",
    "wbp",
    "wed",
    "wer",
    "wim",
    "wiu",
    "wiv",
    "wln",
    "wmt",
    "wmw",
    "wnc",
    "wnu",
    "wol",
    "wos",
    "wrk",
    "wrs",
    "wsk",
    "wuu",
    "wuv",
    "xal",
    "xav",
    "xbi",
    "xed",
    "xer",
    "xho",
    "xir",
    "xla",
    "xmf",
    "xnj",
    "xnn",
    "xno",
    "xnr",
    "xon",
    "xri",
    "xsb",
    "xsi",
    "xta",
    "xtd",
    "xtm",
    "xtn",
    "xuo",
    "yaa",
    "yad",
    "yal",
    "yam",
    "yao",
    "yap",
    "yaq",
    "yby",
    "ycn",
    "yid",
    "yka",
    "yle",
    "yml",
    "yom",
    "yon",
    "yor",
    "yrb",
    "yre",
    "yss",
    "yua",
    "yue",
    "yuj",
    "yut",
    "yuw",
    "yva",
    "zaa",
    "zab",
    "zac",
    "zad",
    "zae",
    "zai",
    "zaj",
    "zam",
    "zao",
    "zaq",
    "zar",
    "zas",
    "zat",
    "zav",
    "zaw",
    "zca",
    "zea",
    "zga",
    "zgh",
    "zia",
    "ziw",
    "zlm",
    "zos",
    "zpc",
    "zpg",
    "zpi",
    "zpl",
    "zpm",
    "zpo",
    "zpq",
    "zpt",
    "zpu",
    "zpv",
    "zpz",
    "zsr",
    "ztg",
    "ztq",
    "zty",
    "zul",
    "zyp",
]

_LICENSE = Licenses.UNKNOWN.value

_LOCAL = False

_URL = "http://sourceforge.net/projects/la-strings/files/Language-Data/LTI-LangID-rel5.txz/download"

_SUPPORTED_TASKS = [Tasks.LANGUAGE_IDENTIFICATION]

_SOURCE_VERSION = "5.0.0"
_SEACROWD_VERSION = "1.0.0"


# TODO: Name the dataset class to match the script name using CamelCase instead of snake_case
class LTILangIDDataset(datasets.GeneratorBasedBuilder):
    """LTI LangID corpus is a dataset for language identification for 1266 languages."""

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=datasets.Version(_SOURCE_VERSION),
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_text",
            version=datasets.Version(_SEACROWD_VERSION),
            description=f"{_DATASETNAME} SEACrowd schema",
            schema="seacrowd_text",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features({"text": datasets.Value("string"), "language": datasets.Value("string")})

        elif self.config.schema == "seacrowd_text":
            features = schemas.text_features(label_names=_ALL_LANGS)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        data_dir = dl_manager.download_and_extract(_URL)
        data_dir = os.path.join(data_dir, "MIL-TALE", "5")

        datasplits_dir = os.path.join(data_dir, "datasplits")
        if not Path(datasplits_dir).exists() or not Path(os.path.join(datasplits_dir, "DONE")).exists():
            # Run provided install.sh to generate train/dev/test splits in "datasplits" folder
            subprocess.call([os.path.join(data_dir, "code", "install.sh"), datasplits_dir])
            with open(os.path.join(datasplits_dir, "DONE"), "w"):
                pass

        train_filepaths = []
        dev_filepaths = []
        test_filepaths = []

        dataset_dir = os.path.join(data_dir, "datasplits")
        print(dataset_dir)

        for lang_id in _ALL_LANGS:
            train_filepaths.append(
                (
                    lang_id,
                    glob(os.path.join(dataset_dir, "train", f"{lang_id}*")),
                )
            )
            dev_filepaths.append(
                (
                    lang_id,
                    glob(os.path.join(dataset_dir, "devtest", f"{lang_id}*")),
                )
            )
            test_filepaths.append(
                (
                    lang_id,
                    glob(os.path.join(dataset_dir, "test", f"{lang_id}*")),
                )
            )

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # Whatever you put in gen_kwargs will be passed to _generate_examples
                gen_kwargs={
                    "all_filepaths": train_filepaths,
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                # Whatever you put in gen_kwargs will be passed to _generate_examples
                gen_kwargs={
                    "all_filepaths": dev_filepaths,
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                # Whatever you put in gen_kwargs will be passed to _generate_examples
                gen_kwargs={
                    "all_filepaths": test_filepaths,
                },
            ),
        ]

    def _generate_examples(self, all_filepaths: List[Path]) -> Tuple[int, Dict]:

        if self.config.schema == "source":
            key = 0
            for lang_id, filepaths in all_filepaths:
                for filepath in filepaths:
                    try:
                        for line in open(filepath):
                            text = line.strip().replace("\u200d", " ").replace("\u200b", " ")
                            yield key, {"text": text, "language": lang_id}
                            key += 1
                    except UnicodeDecodeError:
                        continue

        elif self.config.schema == "seacrowd_text":
            key = 0
            for lang_id, filepaths in all_filepaths:
                for filepath in filepaths:
                    try:
                        for line in open(filepath):
                            text = line.strip().replace("\u200d", " ").replace("\u200b", " ")
                            yield key, {"id": f"{filepath.split('/')[-1]}_{key}", "text": text, "label": lang_id}
                            key += 1
                    except UnicodeDecodeError:
                        continue



_LANG_CONFIG = {
  "ace": {
    "name": "Acehnese",
    "source_subset": "ace"
  },
  "ban": {
    "name": "Balinese",
    "source_subset": "ban"
  },
  "bcl": {
    "name": "Central Bicolano",
    "source_subset": "bcl"
  },
  "bjn": {
    "name": "Banjarese",
    "source_subset": "bjn"
  },
  "bug": {
    "name": "Buginese",
    "source_subset": "bug"
  },
  "cbk": {
    "name": "Chavacano",
    "source_subset": "cbk-zam"
  },
  "gor": {
    "name": "Gorontalo",
    "source_subset": "gor"
  },
  "ilo": {
    "name": "Ilokano",
    "source_subset": "ilo"
  },
  "ind": {
    "name": "Indonesian",
    "source_subset": "id"
  },
  "jav": {
    "name": "Javanese",
    "source_subset": "jv"
  },
  "khm": {
    "name": "Khmer",
    "source_subset": "km"
  },
  "lao": {
    "name": "Lao",
    "source_subset": "lo"
  },
  "mad": {
    "name": "Madurese",
    "source_subset": "mad"
  },
  "map_bms": {
    "name": "Banyumasan (Dialect of Javanese)",
    "source_subset": "map-bms"
  },
  "min": {
    "name": "Minangkabau",
    "source_subset": "min"
  },
  "mnw": {
    "name": "Mon",
    "source_subset": "min"
  },
  "mya": {
    "name": "Burmese",
    "source_subset": "my"
  },
  "nia": {
    "name": "Nias",
    "source_subset": "nia"
  },
  "pag": {
    "name": "Pangasinan",
    "source_subset": "pag"
  },
  "pam": {
    "name": "Kapampangan",
    "source_subset": "pam"
  },
  "shn": {
    "name": "Shan",
    "source_subset": "shn"
  },
  "sun": {
    "name": "Sundanese",
    "source_subset": "su"
  },
  "tet": {
    "name": "Tetum",
    "source_subset": "tet"
  },
  "tgl": {
    "name": "Tagalog",
    "source_subset": "tl"
  },
  "tha": {
    "name": "Thai",
    "source_subset": "th"
  },
  "vie": {
    "name": "Vietnamese",
    "source_subset": "vi"
  },
  "war": {
    "name": "Waray",
    "source_subset": "war"
  }
}



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

'''
This is a dataset containing pdfs scraped from 735 gov.my websites.
'''

from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@article{malaysua_ai_government,
  author    = {{Malaysia-AI}},
  title     = {Crawl Malaysian Government},
  year      = {2023},  % Change to the relevant year if known
  url       = {https://huggingface.co/datasets/malaysia-ai/crawl-my-website
}
"""

_DATASETNAME = "malaysia_ai_government"

_DESCRIPTION = """\
This is a dataset containing pdfs scraped from 735 gov.my websites.
It consists of thousands of the unedited text, a link to the URL where the website was retrieved, and the name of the pdf.
"""

_HOMEPAGE = "https://huggingface.co/datasets/malaysia-ai/crawl-my-website"

_LANGUAGES = ["zlm"]

_LICENSE = Licenses.APACHE_2_0.value

_LOCAL = False


_URLS = {
    _DATASETNAME: "https://huggingface.co/datasets/malaysia-ai/crawl-my-website",
}

_SUPPORTED_TASKS = [Tasks.SELF_SUPERVISED_PRETRAINING]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


_SUBSETS = ["gov_my", "govdocs", "muftiwp_gov_my", "myjms_mohe_gov_my"]


class MalaysiaAIGovernmentDataset(datasets.GeneratorBasedBuilder):
    """Thousands of the unedited text for Malay."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"malaysia_ai_government_{subset}_source",
            version=datasets.Version(_SOURCE_VERSION),
            description=f"malaysia_ai_government_{subset} source schema",
            schema="source",
            subset_id=f"{subset}",
        )
        for subset in _SUBSETS
    ] + [
        SEACrowdConfig(
            name=f"malaysia_ai_government_{subset}_seacrowd_ssp",
            version=datasets.Version(_SEACROWD_VERSION),
            description=f"malaysia_ai_government_{subset} SEACrowd schema",
            schema="seacrowd_ssp",
            subset_id=f"{subset}",
        )
        for subset in _SUBSETS
    ]

    DEFAULT_CONFIG_NAME = "malaysia_ai_government_gov_my_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "file": datasets.Value("string"),
                    "text": datasets.Value("string"),
                    "url": datasets.Value("string"),
                }
            )

        # For example seacrowd_kb, seacrowd_t2t
        elif self.config.schema == "seacrowd_ssp":
            features = schemas.ssp_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""

        urls = _URLS[_DATASETNAME]
        subset = self.config.subset_id
        # data_dir = dl_manager.download_and_extract(urls)
        # dl_manager not used since dataloader uses HF 'load_dataset'

        # Not all datasets have predefined canonical train/val/test splits.
        # If your dataset has no predefined splits, use datasets.Split.TRAIN for all of the data.

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # Whatever you put in gen_kwargs will be passed to _generate_examples
                gen_kwargs={
                    "filepath": urls,
                    "split": "train",
                    "subset": subset,
                },
            )
        ]

    def _generate_examples(self, filepath: Path, split: str, subset: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""

        subset_file = subset.replace("_", ".") + ".jsonl"

        if "myjms_mohe_gov_my" in subset:
            # the last subset has jsonl in the wrong format, actually it's a txt file
            data = datasets.load_dataset("text", data_files=filepath + "/resolve/main/" + subset_file, split="train")
        else:
            data = datasets.load_dataset("/".join(filepath.split("/")[-2:]), split="train", data_files={"train": subset_file})

        for key, sample in enumerate(data):
            if self.config.schema == "source":
                yield key, {
                    "file": sample["file"] if "file" in sample else None,
                    "text": sample["text"] if "text" in sample else sample["body"],
                    "url": sample["url"] if "url" in sample else None,
                }

            elif self.config.schema == "seacrowd_ssp":
                yield key, {
                    "id": key,
                    "text": sample["text"] if "text" in sample else sample["body"],
                }



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
from pathlib import Path
from typing import Dict, List, Tuple

try:
    import audiosegment
except:
    print("Install the `audiosegment` package to use.")

try:
    import textgrid
except:
    print("Install the `textgrid` package to use.")

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@INPROCEEDINGS{
10337314,
author={Rahim, Mohd Zulhafiz and Juan, Sarah Samson and Mohamad, Fitri Suraya},
booktitle={2023 International Conference on Asian Language Processing (IALP)},
title={Improving Speaker Diarization for Low-Resourced Sarawak Malay Language Conversational Speech Corpus},
year={2023},
pages={228-233},
keywords={Training;Oral communication;Data models;Usability;Speech processing;Testing;Speaker diarization;x-vectors;clustering;low-resource;auto-labeling;pseudo-labeling;unsupervised},
doi={10.1109/IALP61005.2023.10337314}}
"""

_DATASETNAME = "sarawak_malay"

_DESCRIPTION = """\
This is a Sarawak Malay conversation data for the purpose of speech technology research. \
At the moment, this is an experimental data and currently used for investigating \
speaker diarization. The data was collected by Faculty of Computer Science and \
Information Technology, Universiti Malaysia Sarawak. The data consists of 38 conversations \
that have been transcribed using Transcriber (see TextGrid folder), where each file \
contains two speakers. Each conversation was recorded by different individuals using microphones \
from mobile devices or laptops thus, different file formats were collected from the data collectors. \
All data was then standardized to mono, 16000Khz, wav format.
"""

_HOMEPAGE = "https://github.com/sarahjuan/sarawakmalay"

_LANGUAGES = ["zlm"]

_LICENSE = Licenses.CC0_1_0.value
_LOCAL = False

_URLS = {
    _DATASETNAME: "https://github.com/sarahjuan/sarawakmalay/archive/refs/heads/main.zip",
}
_SUPPORTED_TASKS = [Tasks.SPEECH_RECOGNITION, Tasks.TEXT_TO_SPEECH]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class SarawakMalayDataset(datasets.GeneratorBasedBuilder):
    """This is experimental Sarawak Malay conversation data collected by \
        Universiti Malaysia Sarawak for speech technology research, \
        specifically speaker diarization. The data includes 38 conversations, \
        each with two speakers, recorded on various devices and then standardized to mono, \
        16000Khz, wav format."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)
    SEACROWD_SCHEMA_NAME = "sptext"

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "speaker_id": datasets.Value("string"),
                    "path": datasets.Value("string"),
                    "audio": datasets.Audio(sampling_rate=16_000),
                    "text": datasets.Value("string"),
                    "metadata": {
                        "malay_text": datasets.Value("string"),
                    },
                }
            )

        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            features = schemas.speech_text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        urls = _URLS[_DATASETNAME]
        data_dir = dl_manager.download_and_extract(urls)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": os.path.join(data_dir, "sarawakmalay-main"),
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        id_counter = 0
        filenames = filter(lambda x: x.endswith(".wav"), os.listdir(f"{filepath}/wav"))
        filenames = map(lambda x: x.replace(".wav", ""), filenames)

        os.makedirs(f"{filepath}/segmented", exist_ok=True)
        for i, filename in enumerate(filenames):
            info = textgrid.TextGrid.fromFile(f"{filepath}/TextGrid/{filename}.TextGrid")
            if len(info) == 3:
                sarawak_conversation, malay_conversation, speakers = info
            else:
                sarawak_conversation, malay_conversation, speakers, _ = info

            audio_file = audiosegment.from_file(f"{filepath}/wav/{filename}.wav").resample(sample_rate_Hz=16000)

            for sarawak_tg, malay_tg, speaker in zip(sarawak_conversation, malay_conversation, speakers):
                start, end, text = sarawak_tg.minTime, sarawak_tg.maxTime, sarawak_tg.mark
                malay_text = malay_tg.mark
                speaker_id = speaker.mark

                start_sec, end_sec = int(start * 1000), int(end * 1000)
                segment = audio_file[start_sec:end_sec]
                segement_filename = f"{filepath}/segmented/{filename}-{round(start, 0)}-{round(end, 0)}.wav"
                segment.export(segement_filename, format="wav")

                if self.config.schema == "source":
                    yield id_counter, {
                        "id": id_counter,
                        "speaker_id": speaker_id,
                        "path": f"{filepath}/wav/{filename}.wav",
                        "audio": segement_filename,
                        "text": text,
                        "metadata": {
                            "malay_text": malay_text,
                        },
                    }

                elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
                    yield id_counter, {"id": id_counter, "speaker_id": speaker_id, "path": f"{filepath}/wav/{filename}.wav", "audio": segement_filename, "text": text, "metadata": {"speaker_age": None, "speaker_gender": None}}

                id_counter += 1



from pathlib import Path
from typing import List, Tuple

import datasets

from seacrowd.sea_datasets.lio_and_central_flores import processing
from seacrowd.sea_datasets.lio_and_central_flores.path_url import _URLS_DICT
from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@misc{alexthesis2018,
  author    = {Alexander Elias},
  title     = {Lio and the Central Flores languages},
  year      = {2018},
  month     = {November},
  address   = {Rapenburg 70, 2311 EZ Leiden},
  school    = {Universiteit Leiden},
  url       = {https://studenttheses.universiteitleiden.nl/handle/1887/69452},
  note      = {Research Master's thesis},
}
"""

_DATASETNAME = "lio_and_central_flores"
_DESCRIPTION = """This dataset is a collection of language resources of Li'o, Ende, Nage, and
So'a which are collected in Ende, Flores, Eastern Nusa Tenggara. This dataset
is the dataset from the research MA thesis by Alexander Elias. Title: Lio and the Central Flores languages
"""
_HOMEPAGE = "https://archive.mpi.nl/tla/islandora/search/alexander%20elias?type=dismax&islandora_solr_search_navigation=0&f%5B0%5D=cmd.Contributor%3A%22Alexander%5C%20Elias%22"
_LICENSE = Licenses.UNKNOWN.value
_LANGUAGES = ["end", "nxe", "ljl"]
LANGUAGES_TO_FILENAME_MAP = {
    "end": "ENDE",
    "nxe": "NAGE",
    "ljl": "LIO",
}
_LOCAL = False

_URLS = _URLS_DICT

_SUPPORTED_TASKS = [Tasks.MACHINE_TRANSLATION]

_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class LioAndCentralFloresDataset(datasets.GeneratorBasedBuilder):
    """This dataset is a collection of language resources of Li'o, Ende, Nage, and So'a"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)
    SEACROWD_SCHEMA_NAME = "t2t"

    dataset_names = sorted([f"{_DATASETNAME}_{lang}" for lang in _LANGUAGES])
    BUILDER_CONFIGS = []
    for name in dataset_names:
        if name.split("_")[4] == "nxe":
            # We only use source schema here for nage ("nxe") because nage dataset only contain wordlist
            # For "nxe" , include a separate configuration to handle word lists. It will be return nage only word list
            source_config = SEACrowdConfig(name=f"{name}_wordlist_source", version=SOURCE_VERSION, description=f"{_DATASETNAME} source schema", schema="source", subset_id=name)
            BUILDER_CONFIGS.append(source_config)
            # Additionally, include a configuration for English word lists in "nxe" datasets. It will be return eng only word corresponding to nage wordlist
            source_config = SEACrowdConfig(name=f"{name}_eng_wordlist_source", version=SOURCE_VERSION, description=f"{_DATASETNAME} source schema", schema="source", subset_id=name)
            BUILDER_CONFIGS.append(source_config)
        else:
            # For other languages, except "nxe", use a standard source schema configuration
            source_config = SEACrowdConfig(name=f"{name}_source", version=SOURCE_VERSION, description=f"{_DATASETNAME} source schema", schema="source", subset_id=name)
            BUILDER_CONFIGS.append(source_config)
            # Additionally, include a SEACrowd schema configuration for other languages
            seacrowd_config = SEACrowdConfig(name=f"{name}_seacrowd_{SEACROWD_SCHEMA_NAME}", version=SEACROWD_VERSION, description=f"{_DATASETNAME} SEACrowd schema", schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}", subset_id=name)
            BUILDER_CONFIGS.append(seacrowd_config)

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            if self.config.name.split("_")[4] == "nxe":
                features = datasets.Features({"id": datasets.Value("string"), "word": datasets.Value("string")})
            else:
                features = datasets.Features({"source_sentence": datasets.Value("string"), "target_sentence": datasets.Value("string"), "source_lang": datasets.Value("string"), "target_lang": datasets.Value("string")})
        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            if self.config.name.split("_")[4] != "nxe":
                features = schemas.text2text_features
        else:
            raise ValueError("Invalid config schema")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""

        lang = self.config.name.split("_")[4]
        if lang in _LANGUAGES:
            filepath = {k: v["text_path"] for k, v in _URLS[LANGUAGES_TO_FILENAME_MAP[lang]].items()}
            paths = dl_manager.download(filepath)
        else:
            raise ValueError("Invalid language name")

        return [datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepath": paths, "language": lang})]

    def _generate_examples(self, filepath: Path, language: str):
        """Yields examples as (key, example) tuples."""

        if language == "nxe" and self.config.schema == "source":
            schema_type = "_".join(self.config.name.split("_")[4:6])
            if schema_type == "nxe_eng":
                words, _ = self._get_word_(filepath)
            else:
                _, words = self._get_word_(filepath)

            for item in words:
                for idx, word in enumerate(item):
                    row = {"id": str(idx), "word": word}
                    yield idx, row

        else:
            if language != "nxe":
                source_data, target_data = self._get_sentence_(filepath)
                for idx, (source, target) in enumerate(zip(source_data, target_data)):
                    if self.config.schema == "source":
                        example = {"source_sentence": source, "target_sentence": target, "source_lang": "eng", "target_lang": language}
                    elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
                        example = {
                            "id": str(idx),
                            "text_1": source,
                            "text_2": target,
                            "text_1_name": "eng",
                            "text_2_name": language,
                        }
                    yield idx, example
            else:
                raise ValueError("Not found 'nxe sentences")

    def _get_sentence_(self, path_dict) -> Tuple[List, List]:
        source_data = []
        target_data = []
        for _, v in path_dict.items():
            with open(v, "r", encoding="utf-8") as f:
                data = f.readlines()
            src, trg = processing.parse_text(data)
            source_data.extend(src)
            target_data.extend(trg)

        return source_data, target_data

    def _get_word_(self, path_dict) -> Tuple[List, List]:
        eng_data, ind_data, nage_data = [], [], []
        for _, v in path_dict.items():
            with open(v, "r", encoding="utf-8") as f:
                data = f.readlines()
            eng_word, ind_word, nage_word = processing.parse_wordlist(data)
            eng_data.append(eng_word)
            ind_data.append(ind_word)
            nage_data.append(nage_word)

        return eng_data, nage_data



import re


import re

def parse_text(data):
    """
    Parses a list of text data into English and Lio language lists.

    Args:
        data (list): A list of strings containing text data.

    Returns:
        tuple: A tuple containing two lists: English language data and Lio language data.
    """
    # Clean the data by removing unwanted characters and formatting
    cleaned_data = [re.sub(r'^\ufeff?\d+\t|\b\d+\)\s*', '', re.sub(r'\t+', '\t', text)).replace('\t\n', '').replace('\ufeff', '') for text in data]

    # Split the data into English and Lio lists
    eng_data, lio_data = [], []
    sublist = []

    for item in cleaned_data:
        # Replace tabs with spaces
        item = item.replace('\t', ' ')
        if item != '':
            sublist.append(item)
        else:
            if sublist:
                # Append sublist to result if not empty
                eng_data.append(sublist[-1])
                lio_data.append(sublist[0])
                sublist = []

    # Append the remaining sublist if any
    if sublist:
        eng_data.append(sublist[-1])
        lio_data.append(sublist[0])

    return eng_data, lio_data

def parse_wordlist(data):
    """
    Parses a word list for translation into English, Indonesian, and Nage languages.

    Args:
        data (list): A list of strings containing word list data.

    Returns:
        tuple: A tuple containing three lists: English words, Indonesian words, and Nage words.
    """
    # Clean the data by removing unwanted characters and formatting
    cleaned_data = [re.sub(r'^\ufeff?\d+\t|\b\d+\)\s*', '', re.sub(r'\t+', '\t', text)).replace('\t\n', '').replace('\ufeff', '') for text in data]
    
    # Remove empty elements
    cleaned_data = [elem for elem in cleaned_data if elem != '']
    
    eng_word = []
    nage_word = []
    ind_word = []
    
    for item in cleaned_data:
        split_data = item.split()
        # make sure that we only get [eng, ind, nage] because the actual wordlist have 
        # added 'comment' column and not all the words have their comments
        if len(split_data) == 3: 
            eng_word.append(split_data[0])
            ind_word.append(split_data[1])
            nage_word.append(split_data[2])

    return eng_word, ind_word, nage_word







import json
from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import jsonlines

from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@article{,
  author    = {Audah, Hanif Arkan and Yuliawati, Arlisa and Alfina, Ika},
  title     = {A Comparison Between SymSpell and a Combination of Damerau-Levenshtein Distance With the Trie Data Structure},
  journal   = {2023 10th International Conference on Advanced Informatics: Concept, Theory and Application (ICAICTA)},
  volume    = {},
  year      = {2023},
  url       = {https://ieeexplore.ieee.org/stamp/stamp.jsp?arnumber=10390399&casa_token=HtJUCIGGlWYAAAAA:q8ll1RWmpHtSAq2Qp5uQAE1NJETx7tUYFZIvTO1IWoaYy4eqFETSsm9p6C7tJwLZBGq5y8zc3A&tag=1},
  doi       = {},
  biburl    = {https://github.com/ir-nlp-csui/saltik?tab=readme-ov-file#references},
  bibsource = {https://github.com/ir-nlp-csui/saltik?tab=readme-ov-file#references}
}
"""

_DATASETNAME = "saltik"
_DESCRIPTION = """\
Saltik is a dataset for benchmarking non-word error correction method accuracy in evaluating Indonesian words.
It consists of 58,532 non-word errors generated from 3,000 of the most popular Indonesian words.
"""
_HOMEPAGE = "https://github.com/ir-nlp-csui/saltik"
_LANGUAGES = ["ind"]
_LICENSE = Licenses.AGPL_3_0.value
_LOCAL = False
_URLS = {
    _DATASETNAME: "https://raw.githubusercontent.com/ir-nlp-csui/saltik/main/saltik.json",
}
_SUPPORTED_TASKS = []
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class Saltik(datasets.GeneratorBasedBuilder):
    """It consists of 58,532 non-word errors generated from 3,000 of the most popular Indonesian words."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            # EX: Arbitrary NER type dataset
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "word": datasets.Value("string"),
                    "errors": [
                        {
                            "typo": datasets.Value("string"),
                            "error_type": datasets.Value("string"),
                        }
                    ],
                }
            )
        else:
            raise NotImplementedError()

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        urls = _URLS[_DATASETNAME]
        file_path = dl_manager.download(urls)
        data = self._read_jsonl(file_path)
        all_words = list(data.keys())
        processed_data = []
        id = 0
        for word in all_words:
            processed_data.append({"id": id, "word": word, "errors": data[word]})
            id += 1
        self._write_jsonl(file_path + ".jsonl", processed_data)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # Whatever you put in gen_kwargs will be passed to _generate_examples
                gen_kwargs={
                    "filepath": file_path + ".jsonl",
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        if self.config.schema == "source":
            i = 0
            with jsonlines.open(filepath) as f:
                for each_data in f.iter():
                    ex = {
                        "id": each_data["id"],
                        "word": each_data["word"],
                        "errors": each_data["errors"],
                    }

                    yield i, ex
                    i += 1

    def _read_jsonl(self, filepath: Path):
        with open(filepath) as user_file:
            parsed_json = json.load(user_file)
            return parsed_json

    def _write_jsonl(self, filepath, values):
        with jsonlines.open(filepath, "w") as writer:
            for line in values:
                writer.write(line)



# coding=utf-8
# Copyright 2024 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
SIB-200 is the largest publicly available topic classification dataset based on Flores-200 covering 205 languages and dialects.
The train/validation/test sets are available for all the 205 languages.
"""

import os
from pathlib import Path
from typing import List, Tuple, Dict

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks, Licenses

_CITATION = """\
@misc{adelani2023sib200,
      title={SIB-200: A Simple, Inclusive, and Big Evaluation Dataset for Topic Classification in 200+ Languages and Dialects},
      author={David Ifeoluwa Adelani and Hannah Liu and Xiaoyu Shen and Nikita Vassilyev and Jesujoba O. Alabi and Yanke Mao and Haonan Gao and Annie En-Shiun Lee},
      year={2023},
      eprint={2309.07445},
      archivePrefix={arXiv},
      primaryClass={cs.CL}
}
"""

_DATASETNAME = "sib_200"

_DESCRIPTION = """\
SIB-200 is the largest publicly available topic classification dataset based on Flores-200 covering 205 languages and dialects.
The train/validation/test sets are available for all the 205 languages.
"""

_HOMEPAGE = "https://github.com/dadelani/sib-200"

_LANGUAGES = [
    "ace",
    "ban",
    "bjn",
    "bug",
    "ceb",
    "ilo",
    "ind",
    "jav",
    "kac",
    "khm",
    "lao",
    "lus",
    "min",
    "mya",
    "pag",
    "shn",
    "sun",
    "tgl",
    "tha",
    "vie",
    "war",
    "zsm",
]

_SUPPORTED_LANGUAGE_CODES = [
    "ace_Arab",
    "ace_Latn",
    "ban_Latn",
    "bjn_Arab",
    "bjn_Latn",
    "bug_Latn",
    "ceb_Latn",
    "ilo_Latn",
    "ind_Latn",
    "jav_Latn",
    "kac_Latn",
    "khm_Khmr",
    "lao_Laoo",
    "lus_Latn",
    "min_Arab",
    "min_Latn",
    "mya_Mymr",
    "pag_Latn",
    "shn_Mymr",
    "sun_Latn",
    "tgl_Latn",
    "tha_Thai",
    "vie_Latn",
    "war_Latn",
    "zsm_Latn",
]

_LICENSE = Licenses.CC_BY_SA_4_0.value

_LOCAL = False

# This can be an arbitrarily nested dict/list of URLs (see below in `_split_generators` method)
_URLS = {
    "base_url": "https://huggingface.co/datasets/Davlan/sib200/raw/main/data"
}

_SUPPORTED_TASKS = [Tasks.TOPIC_MODELING]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"

_SEACROWD_SCHEMA = f"seacrowd_text"


def _sib_config_constructor(lang: str, schema: str = _SEACROWD_SCHEMA, version: str = _SEACROWD_VERSION) -> SEACrowdConfig:
    return SEACrowdConfig(
        name=f"{_DATASETNAME}_{lang}_{schema}",
        version=version,
        description=f"SIB-200 {schema} schema",
        schema=schema,
        subset_id=f"SIB-200 {lang}",
    )


class Sib200Dataset(datasets.GeneratorBasedBuilder):
    """
    SIB-200 is the largest publicly available topic classification dataset based on Flores-200 covering 205 languages and dialects.
    The train/validation/test sets are available for all the 205 languages.
    """

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    def _populate_configs():
        configs = [_sib_config_constructor(lang, schema="source", version=_SOURCE_VERSION) for lang in _SUPPORTED_LANGUAGE_CODES] + [_sib_config_constructor(lang, schema=_SEACROWD_SCHEMA, version=_SEACROWD_VERSION) for lang in _SUPPORTED_LANGUAGE_CODES]

        all_lang_source_config = SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=_SOURCE_VERSION,
            description=f"SIB-200 source schema",
            schema="source",
            subset_id=f"SIB-200 SEA",
        )

        all_lang_t2t_config = SEACrowdConfig(
            name=f"{_DATASETNAME}_{_SEACROWD_SCHEMA}",
            version=_SEACROWD_VERSION,
            description=f"SIB-200 {_SEACROWD_SCHEMA} schema",
            schema=_SEACROWD_SCHEMA,
            subset_id=f"SIB-200 SEA",
        )

        configs.append(all_lang_source_config)
        configs.append(all_lang_t2t_config)
        return configs

    BUILDER_CONFIGS = _populate_configs()

    DEFAULT_CONFIG_NAME = "sib_200_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "index_id": datasets.Value("int64"),
                    "text": datasets.Value("string"),
                    "category": datasets.Value("string"),
                }
            )

        elif self.config.schema == "seacrowd_text":
            features = schemas.text_features(["geography", "science/technology", "health", "travel", "entertainment", "politics", "sports"])

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        # dl_manager not used since dataloader uses HF 'load_dataset'
        lang = self.config.subset_id.split(" ")[-1]
        if lang in _SUPPORTED_LANGUAGE_CODES:
            train_paths = [Path(dl_manager.download_and_extract(f"{_URLS['base_url']}/{lang}/train.tsv"))]
            valid_paths = [Path(dl_manager.download_and_extract(f"{_URLS['base_url']}/{lang}/dev.tsv"))]
            test_paths = [Path(dl_manager.download_and_extract(f"{_URLS['base_url']}/{lang}/test.tsv"))]
            lang_codes = [lang]
        elif lang == "SEA":
            train_paths, valid_paths, test_paths, lang_codes = [], [], [], []
            for lang in _SUPPORTED_LANGUAGE_CODES:                
                train_paths.append(Path(dl_manager.download_and_extract(f"{_URLS['base_url']}/{lang}/train.tsv")))
                valid_paths.append(Path(dl_manager.download_and_extract(f"{_URLS['base_url']}/{lang}/dev.tsv")))
                test_paths.append(Path(dl_manager.download_and_extract(f"{_URLS['base_url']}/{lang}/test.tsv")))
                lang_codes.append(lang)
        else:
            raise ValueError(f"Language {lang} not a SEA language in the dataset")
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN, 
                gen_kwargs={"file_paths": train_paths, "lang_codes": lang_codes}
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION, 
                gen_kwargs={"file_paths": valid_paths, "lang_codes": lang_codes}
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST, 
                gen_kwargs={"file_paths": test_paths, "lang_codes": lang_codes}
            )
        ]

    def _generate_examples(self, file_paths: List[str], lang_codes: List[str]) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        index = 0
        for file_path, lang_code in zip(file_paths, lang_codes):
            lang_df = pd.read_csv(file_path, sep='\t')
            for row in lang_df.itertuples():
                if self.config.schema == "source":
                    example = {
                        "index_id": row.index_id,
                        "text": row.text,
                        "category": row.category,
                    }

                elif self.config.schema == "seacrowd_text":
                    example = {
                        "id": f'{lang_code}_{row.index_id}',
                        "text": row.text,
                        "label": row.category,
                    }
                yield index, example
                index += 1



import os
from pathlib import Path, PureWindowsPath
from typing import Dict, List, Tuple

try:
    import cv2
except:
    print("Install the `cv2` package to use.")
import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@article{tupal4476867fsl105,
  title={FSL105: The Video Filipino Sign Language Sign Database of Introductory 105 FSL Signs},
  author={Tupal, Isaiah Jassen Lizaso and Melvin, Cabatuan K},
  journal={Available at SSRN 4476867}
}
"""

_DATASETNAME = "fsl_105"

_DESCRIPTION = """\
FSL-105 is a video dataset for 105 different Filipino Sign Language (FSL) signs.
Each sign is categorized into one of 10 categories and is each represented by approximately 20 four-second video samples.
Signs were performed by adult deaf FSL signers on a blank blue background and reviewed by an FSL expert.
"""

_HOMEPAGE = "https://data.mendeley.com/datasets/48y2y99mb9/2"

_LICENSE = Licenses.CC_BY_4_0.value

_LOCAL = False

_URLS = {
    "clips": "https://prod-dcd-datasets-public-files-eu-west-1.s3.eu-west-1.amazonaws.com/de95a3c3-02f4-4a3f-9a9e-ce2371160275",
    "train": "https://prod-dcd-datasets-public-files-eu-west-1.s3.eu-west-1.amazonaws.com/09c71779-3a2a-4c98-8d9b-0ef74f54d92a",
    "test": "https://prod-dcd-datasets-public-files-eu-west-1.s3.eu-west-1.amazonaws.com/39af8117-6b44-47b9-a551-0bdc40837295",
}

_LANGUAGES = ["psp"]

_SUPPORTED_TASKS = [Tasks.VIDEO_TO_TEXT_RETRIEVAL, Tasks.VIDEO_CAPTIONING]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class FSL105Dataset(datasets.GeneratorBasedBuilder):
    """
    FSL-105 is a video dataset for 105 different Filipino Sign Language (FSL) signs.
    Each sign is categorized into one of 10 categories and is each represented by approximately 20 four-second video samples.
    Signs were performed by adult deaf FSL signers on a blank blue background and reviewed by an FSL expert.
    """

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_vidtext",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema="seacrowd_vidtext",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    category = [
        "CALENDAR",
        "COLOR",
        "DAYS",
        "DRINK",
        "FAMILY",
        "FOOD",
        "GREETING",
        "NUMBER",
        "RELATIONSHIPS",
        "SURVIVAL",
    ]

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "video_path": datasets.Value("string"),
                    "text": datasets.Value("string"),
                    "labels": datasets.ClassLabel(names=self.category),
                    "metadata": {
                        "resolution": {
                            "width": datasets.Value("int64"),
                            "height": datasets.Value("int64"),
                        },
                        "duration": datasets.Value("float32"),
                        "fps": datasets.Value("float32"),
                    },
                }
            )

        elif self.config.schema == "seacrowd_vidtext":
            features = schemas.video_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""

        clips = dl_manager.download_and_extract(_URLS["clips"])
        train = dl_manager.download_and_extract(_URLS["train"])
        test = dl_manager.download_and_extract(_URLS["test"])

        train_df = pd.read_csv(train)
        test_df = pd.read_csv(test)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": {
                        "clips": clips,
                        "data": train_df,
                    },
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": {"clips": clips, "data": test_df},
                    "split": "test",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""

        for key, example in filepath["data"].iterrows():
            video = cv2.VideoCapture(os.path.join(filepath["clips"], PureWindowsPath(example["vid_path"]).as_posix()))
            fps = video.get(cv2.CAP_PROP_FPS)
            frame_count = video.get(cv2.CAP_PROP_FRAME_COUNT)
            duration = frame_count / fps
            vid_width = video.get(cv2.CAP_PROP_FRAME_WIDTH)
            vid_height = video.get(cv2.CAP_PROP_FRAME_HEIGHT)

            if self.config.schema == "source":
                yield key, {
                    "id": str(key),
                    "video_path": os.path.join(filepath["clips"], example["vid_path"]),
                    "text": example["label"],
                    "labels": example["category"],
                    "metadata": {
                        "resolution": {
                            "width": vid_width,
                            "height": vid_height,
                        },
                        "duration": duration,
                        "fps": fps,
                    },
                }
            elif self.config.schema == "seacrowd_vidtext":
                yield key, {
                    "id": str(key),
                    "video_path": os.path.join(filepath["clips"], example["vid_path"]),
                    "text": example["label"],
                    "metadata": {
                        "resolution": {
                            "width": vid_width,
                            "height": vid_height,
                        },
                        "duration": duration,
                        "fps": fps,
                    },
                }



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@INPROCEEDINGS{8584876,
  author={Joseph, Ferdin Joe John and Anantaprayoon, Panatchakorn},
  booktitle={2018 International Conference on Information Technology (InCIT)},
  title={Offline Handwritten Thai Character Recognition Using Single Tier Classifier and Local Features},
  year={2018},
  volume={},
  number={},
  pages={1-4},
  abstract={Handwritten character recognition is a conversion process of handwriting into machine-encoded text. Currently,
  several techniques and methods are proposed to enhance accuracy of handwritten character recognition for many languages
  spoken across the globe. In this project, a local feature-based approach is proposed to enhance the accuracy of handwritten
  offline character recognition for Thai alphabets. In the experiment, through MATLAB, 100 images for each class of Thai
  alphabets are collected and k-fold cross validation is applied to manage datasets to train and test. A gradient invariant
  feature set consisting of LBP and shape features is extracted. The classification is operated by using query matching based
  on Euclidean distance. The accuracy would be the percentage of correct classification for each class. For the result, the
  highest accuracy is 68.96% which has 144-bit shape features and uniform pattern LBP for the features.},
  keywords={Character recognition;Feature extraction;Shape;Genetic algorithms;Matlab;Gray-scale;Optical character recognition
  software;Offline Character Recognition;Local Binary Pattern;Thai Handwriting},
  doi={10.23919/INCIT.2018.8584876},
  ISSN={},
  month={Oct},
  url={https://ieeexplore.ieee.org/document/8584876}}

"""

_DATASETNAME = "kvis_th_ocr"

_DESCRIPTION = """\
The KVIS Thai OCR Dataset contains scanned handwritten version of all 44 Thai characters obtained from 27 individuals. It
consisted of 1079 images from 44 classes (letters). This dataset consists of all Thai consonants with different writing
styles of various people from ages between 16 and 75. Vowels and intonation are not taken into consideration for the dataset
collected.
"""

_HOMEPAGE = "https://data.mendeley.com/datasets/8nr3pbdk5c/1"

_LANGUAGES = ["tha"]

_LICENSE = Licenses.CC_BY_4_0.value

_LOCAL = False

_URLS = "https://prod-dcd-datasets-cache-zipfiles.s3.eu-west-1.amazonaws.com/8nr3pbdk5c-1.zip"

_SUPPORTED_TASKS = [Tasks.OPTICAL_CHARACTER_RECOGNITION]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class KVISThaiOCRDataset(datasets.GeneratorBasedBuilder):
    """
    KVIS Thai OCR is a dataset for optical character recognition for Thai characters from https://data.mendeley.com/datasets/8nr3pbdk5c/1.
    """

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)
    labels = ["ก", "ข", "ฃ", "ค", "ฅ", "ฆ", "ง", "จ", "ฉ", "ช", "ซ", "ฌ", "ญ", "ฎ", "ฏ", "ฐ", "ฑ", "ฒ", "ณ", "ด", "ต", "ถ", "ท", "ธ", "น", "บ", "ป", "ผ", "ฝ", "พ", "ฟ", "ภ", "ม", "ย", "ร", "ล", "ว", "ศ", "ษ", "ส", "ห", "ฬ", "อ", "ฮ"]

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=datasets.Version(_SOURCE_VERSION),
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_imtext",
            version=datasets.Version(_SOURCE_VERSION),
            description=f"{_DATASETNAME} SEACrowd schema",
            schema="seacrowd_imtext",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "file_path": datasets.Value("string"),
                    "character": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_imtext":
            features = schemas.image_text_features(label_names=self.labels)
        else:
            raise ValueError(f"Invalid schema: {self.config.schema}")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """
        Returns SplitGenerators.
        """

        dir = dl_manager.download_and_extract(_URLS)
        path = dl_manager.extract(os.path.join(dir, "KVIS TOCR Dataset.zip"))

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "path": path,
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, path: Path, split: str) -> Tuple[int, Dict]:
        """
        Yields examples as (key, example) tuples.
        """

        idx = 0
        path = list(os.walk(path))
        for directory in path[1:]:
            label = directory[0][-3:-2]
            for file in directory[2]:
                file_extension = str(file[-3:])
                if file_extension == "jpg":
                    file_id = str(file[:-4])
                    file_path = os.path.join(directory[0], file)
                    if self.config.schema == "source":
                        data = {
                            "id": file_id,
                            "file_path": file_path,
                            "character": label,
                        }
                        yield idx, data
                        idx += 1
                    elif self.config.schema == "seacrowd_imtext":
                        data = {
                            "id": file_id,
                            "image_paths": [file_path],
                            "texts": "",
                            "metadata": {
                                "context": "",
                                "labels": [label],
                            },
                        }
                        yield idx, data
                        idx += 1
                    else:
                        raise ValueError(f"Invalid schema: {self.config.schema}")



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@article{malaysian-dataset,
  title = {Malaysian-Dataset},
  url = {https://github.com/mesolitica/malaysian-dataset},
}
"""

_DATASETNAME = "chatgpt_malaysian_open_qa"

_DESCRIPTION = """\
This is a synthetic Malaysian Open QA dataset generated using ChatGPT3.5 on MS Wikipedia, MS Common Crawl, and Malaysia Hansard.
Subsets include `common-crawl-qa` (69k rows), `hansard-qa` (42k rows), and `wikipedia-qa` (44k rows).
"""

_HOMEPAGE = "https://huggingface.co/datasets/mesolitica/chatgpt-malaysian-open-qa"

_LANGUAGES = ["zlm"]

_LICENSE = Licenses.CC_BY_NC_2_0.value

_LOCAL = False

_URLS = {
    "common_crawl_qa": "https://huggingface.co/datasets/mesolitica/chatgpt-malaysian-open-qa/resolve/main/common-crawl-qa.jsonl",
    "hansard_qa": "https://huggingface.co/datasets/mesolitica/chatgpt-malaysian-open-qa/resolve/main/hansard-qa.jsonl",
    "wikipedia_qa": "https://huggingface.co/datasets/mesolitica/chatgpt-malaysian-open-qa/resolve/main/wikipedia-qa.jsonl",
}

_SUPPORTED_TASKS = [Tasks.QUESTION_ANSWERING]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class ChatGPTMalaysianOpenQADataset(datasets.GeneratorBasedBuilder):
    """
    ChatGPT Malaysian Open QA Dataset is a Malaysian QA dataset from https://huggingface.co/datasets/mesolitica/chatgpt-malaysian-open-qa.
    """

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=datasets.Version(_SOURCE_VERSION),
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_qa",
            version=datasets.Version(_SEACROWD_VERSION),
            description=f"{_DATASETNAME} SEACrowd schema",
            schema="seacrowd_qa",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "paragraph": datasets.Value("string"),
                    "qa": datasets.Value("string"),
                    "url": datasets.Value("string"),
                    "source": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_qa":
            features = schemas.qa_features
            features["meta"] = {
                "url": datasets.Value("string"),
                "source": datasets.Value("string"),
            }
        else:
            raise ValueError(f"Invalid schema: '{self.config.schema}'")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """
        Returns SplitGenerators.
        """
        paths = dl_manager.download_and_extract(_URLS)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepaths": paths,
                    "split": "train",
                },
            )
        ]

    def _generate_examples(self, filepaths: Path, split: str) -> Tuple[int, Dict]:
        """
        Yields examples as (key, example) tuples.
        """

        idx = 0
        for name, file in filepaths.items():
            with open(file, "r") as f:
                data = list(map(json.loads, f))
                if self.config.schema == "source":
                    for d in data:
                        x = {k: v if v != "" and k in self.info.features else None for k, v in d.items()}
                        x["url"] = d.get("url", None)
                        x["source"] = name
                        yield idx, x
                        idx += 1
                elif self.config.schema == "seacrowd_qa":
                    for d in data:
                        for q in d["qa"]["qa"]:
                            x = {
                                "id": idx,
                                "question_id": idx,
                                "document_id": idx,
                                "question": q["question"],
                                "type": "extractive",
                                "choices": [],
                                "context": d["paragraph"],
                                "answer": [q["answer"]],
                                "meta": {
                                    "url": d.get("url", None),
                                    "source": name,
                                },
                            }
                            yield idx, x
                            idx += 1
                else:
                    raise ValueError(f"Invalid schema: '{self.config.schema}'")
